//////////////////////////////////////////////////////////////
//Wolfshade MUD server
//Copyright (C) 1999 Demetrius and John Comes
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////
/*
*
*
*  CCharIntermediate
*
*  Class will be used as a go between or messaging class
*  between csocket to cgame.
*/

#ifndef _CHARINTERMEDIATE_H_
#define _CHARINTERMEDIATE_H_


class MySocket;
namespace db { class Account; }

//For a new character we will have this class Set up add variables
//properly but advance level will be done in CCharacter class!


class CCharIntermediate : public CCharacterAttributes
{

public:
  			~CCharIntermediate();
  			CCharIntermediate(CCharacter *pCh);
  			CCharIntermediate(CGame *,sPlayerDescriptor *desc);
  			CCharIntermediate(CCharIntermediate *ch);

  bool 			IsPlaying(){return m_pDesc->IsPlaying();}
  bool 			IsLinkDead() {return m_pDesc->IsLinkDead();}
  bool 			ShouldKillIntermediate() {return m_pDesc->ShouldKillIntermediate();}
  void 			SendMenu() {SendToChar(m_strMenu);}
  short 		GetClass() {return m_pAttributes->GetClass();}

  //All functions to move through states get a pointer to MySocket class
  //so that we can access it's private members we are a friend.

  // Initial state = STATE_TITLE_QUESTION

  void 			StateTitle();		// -> STATE_GET_NAME
  void 			StateGetName();		// -> STATE_GET_SAVED_PASSWORD
  						// -> STATE_CONFIRM_NEW_PLAYER_NAME
  void 			StateConfirmName();	// -> STATE_GET_NAME
  						// -> STATE_GET_PASSWORD
  void 			StateGetPassWord();	// -> STATE_CONFIRM_NEW_PASSWORD
  void 			StateConfirmPassword();	// -> STATE_PICK_RACE
  						// -> STATE_LINK_DEAD <klick>
  void 			StatePickRace();	// -> STATE_PICK_CLASS
  void 			StatePickClass();	// -> STATE_ROLL_CHAR <Illithid>
  						// -> STATE_PICK_SEX
  void 			StatePickSex();		// -> STATE_ROLL_CHAR
  void 			StateRollChar();	// -> STATE_BONUS
  void 			StateBonus();		// -> STATE_KEEP_CHAR
  void 			StateMOTD();		// -> STATE_MENU
  void 			StateMenu();		// '0' -> STATE_LINK_DEAD
  						// '1' -> STATE_PLAYING
  						// '2' -> STATE_CHANGE_PASSWORD
  						// '3' -> STATE_ENTER_DESCRIPTION
  						// '4' -> STATE_CONFIRM_DELETE
  void 			StateGetSavedPassword();// -> STATE_KILL_INTERMEDIATE <take over body>
  						// -> STATE_LINK_DEAD <playing, wrong pw>
  						// -> STATE_MOTD
  void 			StateKeepChar();	// -> STATE_PICK_RACE
  						// -> STATE_MOTD
  void 			StateConfirmDeletion();	// -> STATE_LINK_DEAD <deleted>
  						// -> STATE_MENU
  void 			StateChangePassword();	// -> STATE_CHANGE_PASSWORD_CONFIRM
  void 			StateConfirmPasswordChange(); // -> STATE_MENU

  // New Stuff
  void			StateGetAccount();	// -> STATE_ACCOUNT_PASSWORD
  						// -> STATE_CONFIRM_NEW_ACCOUNT_NAME
  void			StateConfirmNewAccountName();
  void			StateGetEmail();
  void			StateAccountMenu();
  void			StateAccountPassword();
  void			StateCreatePlayerQuestion();
  void			StateNewNameQuestion();

  // End of States

  void 			ExecuteState();
  bool 			HasCommand();
  //don't delete descriptor pointer!
  sPlayerDescriptor *	m_pDesc;
//  void 			SendToChar(const char *strSend);
  void 			SendToChar(MyString strSend) const;

protected:
  short 		m_nBonus;
  int 			m_nTimeSinceLastCmd; //used for cutting connection
  bool 			IsLegalName();
  void 			SendClassList(short nRace);

private:
  void 			InitStatics();
  void 			BootFileStrings();
  void 			SendToSocket(const MyString &str) const;
  void			SendPrompt(int);
  void			ShowAccountMenu();
  void			SendPassword(db::Account &);
  bool			ValidEmail(char *);
  /*static*/ void 	MakeRaceList(MyString &);
  static void 		ReadNotAllowedNames();
  static const char *	m_strMenu;
  static MyMap<MyString,bool> NamesNotAllowed;
  static bool 		m_bInited;
  static const bool 	Matrix[TOTAL_RACES][TOTAL_CLASSES];
  static const char *	m_strBonus;
//  static MyString 	m_strRaceList;
  static const int 	VOID_TIME;
  static MyString 	TitleScreen;
  static MyString 	AnsiTitleScreen;
  void 			ChangeState(unsigned short int nState) {m_pDesc->ChangeState(nState);}
  //
  MyString		m_strAccountName;
};
#endif

