//////////////////////////////////////////////////////////////
//Wolfshade MUD server
//Copyright (C) 1999 Demetrius and John Comes
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////
/*
*
*	CCharacterAttributes
*
*	A simple class to hold most character attributes
*	so that we can derive CCharIntermediate from it as
*	well as CCharacter
*/

#ifndef _CHARACTERATTRIBUTES_H_
#define _CHARACTERATTRIBUTES_H_


class CGame;
namespace db { class Player; }

class CCharacterAttributes
{
  friend struct sSaveChar;
  struct sFragInfo
  {
  public:
  	static short ALIEN;
  	static short GOOD;
  	static short EVIL;
  	static short NEUTRAL;
  public:
  	MyString m_strName;
  	short m_nRace,
  		m_nClass;
  	float m_fFrags;
  	byte m_nRaceWar;
  	//actually test the reserve because we want sort in asc order not decending
  	bool operator < (const sFragInfo &r) const {return (m_fFrags > r.m_fFrags);}
  	bool operator !=(const char *s) {return strcmp((char *)m_strName,s);}
  	bool operator ==(const char *s) {return !strcmp((char *)m_strName,s);}
  	sFragInfo (const sSaveChar SaveChar)
  	{
  		m_strName = SaveChar.m_strName;
  		m_fFrags = SaveChar.m_fFrags;
  		m_nRace = SaveChar.m_SavedAttributes.m_nRace;
  		m_nClass = SaveChar.m_SavedAttributes.m_nChClass;
  		if(SaveChar.m_SavedAttributes.m_nRace==RACE_ILLITHID ||
  			SaveChar.m_SavedAttributes.m_nRace==RACE_THRIKREEN)
  		{
  			m_nRaceWar = ALIEN;
  		}
  		else if(SaveChar.m_nAlignment>GOOD_ALIGN)
  		{
  			m_nRaceWar = GOOD;
  		}
  		else if(SaveChar.m_nAlignment<EVIL_ALIGN)
  		{
  			m_nRaceWar = EVIL;
  		}
  		else
  		{
  			m_nRaceWar = NEUTRAL;
  		}
  	}
  };

public:
  virtual 		~CCharacterAttributes();

  static const int 	PREFERS_NO_HASSLE;
  static const int 	PREFERS_TELLS_WHEN_SWITCHED;
  static const int 	PREFERS_COLOR;
  static const int 	PREFERS_AUTO_EXIT;
  static const int 	PREFERS_BRIEF;
  static const int 	PREFERS_MAP_INSIDE;
  static const int 	PREFERS_MAP_OUTSIDE;
  static const int 	PREFERS_TELLS;
  static const int 	PREFERS_GOD_LIGHT;
  static const int 	PREFERS_PETITION;
  static const int 	PREFERS_GOD_CHAT;
  static const int 	PREFERS_GOD_LOG_DEATHS;
  static const int 	PREFERS_GOD_LOG_ZONES;
  static const int 	PREFERS_GOD_SEES_ALL;
  static const int 	PREFERS_GOD_CONNECTION_LOG;
  static const int 	PREFERS_GOD_ON_WHO;
  static const int 	PREFERS_COMPACT;
  static const int 	PREFERS_OOC;
  static const int 	PREFERS_EXTRA_RETURNS;
  static const int 	PREFRES_NO_PAGING;
  static const int 	PREFERS_GOD_INVIS;
  static short 		FindSkill(MyString &strSkill);
  static MyString 	FindSkill(short nSkill);
  int 			GetMaxHits() {return m_nMaxHpts;}
  void 			GetFragList(MyString &str);
  stat_type 		GetCharisma() {return m_pAttributes->GetStat(CHA);}
  bool 			IsOfSphere(int nSphere) const {return ((m_nSpellSphere & nSphere)==nSphere) ? true : false;}
  bool 			HasSkillSet(int nSphere) const {return ((m_nSkillSphere & nSphere)==nSphere) ? true : false;}
  short 		GetAlignment() const {return m_nAlignment;}
  bool 			IsEvil() const {return (m_nAlignment < EVIL_ALIGN);}
  bool 			IsNeutral() const {return ((m_nAlignment >= EVIL_ALIGN) && (m_nAlignment <= GOOD_ALIGN));}
  bool 			IsGood() const {return (m_nAlignment > GOOD_ALIGN);}
  bool 			IsAffectedBy(short nAffect) const {return m_Affects[nAffect].HasAffect();}
  long 			GetLastRentRoom() {return m_lLastRentRoom;}
  MyString &		GetPassword() {return m_strPassword;}
  bool 			IsNewbie() const {return m_nLevel==0;}
  short int 		GetLevel() const {return m_nLevel;}
  MyString 		GetName() const {return m_strName;}
  MyString 		GetAlias() const {return m_strAlias;}
  MyString 		GetDescription() const {return m_strDescription;}
  MyString 		GetLongDescription() const {return m_strLongDescription;}
  short 		GetRace() const {return m_pAttributes->GetRace();}
  short 		GetSize() const {return m_pAttributes->GetSize();}
  short 		GetClass() const {return m_pAttributes->GetClass();}
  short 		GetGod() const {return m_nWorshipedGod;}
  int 			GetCurrentHpts() {return m_nCurrentHpts;}
  bool 			InPosition(int nPos) const {return (m_nPosition & nPos) ? true : false;}
  MyString 		GetTitle() {return m_strTitle;}
  void 			SetFuguePlane() {m_lLastRentRoom = FUGUE_PLANE;}
  short 		GetFugueTime() {return m_nFuguePlaneTime;}
  bool 			IsNewCharacter() {long lPos;return !m_PlayerFilePos.Lookup(m_strName,lPos);}
  bool 			ShouldBeImp() {m_PlayerFile.seekg(0,std::ios::end);return 0==m_PlayerFile.tellg();}
  void 			SetFugueTime(short nTime) {m_nFuguePlaneTime = nTime;}
  //do not make Prefers (m_nPreference & nPreference)==nPreference....THAT IS WRONG!
  bool 			Prefers(int nPreference) const {return (m_nPreference & nPreference) ? true : false;}
  void 			TogPreference(int nPref) {(m_nPreference & nPref) ? m_nPreference &=~nPref : m_nPreference |= nPref;}
  bool 			IsRace(short nRace) const {return m_pAttributes->IsRace(nRace);}
  void 			AdjustCurrentMoves(short nAdj);
  void 			AdjustMaxMoves(short nAdj);
  const char *		GetPositionNames(bool bForScore=true);
  bool 			DeleteChar();
  void 			SetMobDesc(MyString strSet) {m_strDescription = strSet;}
  void 			SetHits(int iSet);
  static bool 		IsSelf(MyString &strName);
  static const char*	GetPronoun(short int nSex);
  static const char*	GetSexName(short int nSex);
  static const char *	GetSizeName(int nSize);
  static int 		GetSavePlayerCount();
  static bool 		ReadNews();
  static bool 		ReadMOTD();
  static void 		CleanUp();

  CGame *		GetGame() { return m_pGame; }
  Database *		GetDatabase();

protected:
  CCharacterAttributes(CMobPrototype &mob,CRoom *pPutInRoom);
  CCharacterAttributes(CCharIntermediate *ch);
  CCharacterAttributes(CCharacterAttributes *pCh);
  CCharacterAttributes(CGame *);
  CCharacterAttributes(CCharacter *pCh);

  static std::fstream 	m_PlayerFile;
  static MyMap<MyString,long> m_PlayerFilePos;
  static MyString 	NEWS;
  static MyString 	MOTD;
  static std::vector<sFragInfo> m_FragList;
  //statics
  static const char *	SexNames[3];
  static const char *	Pronouns[3];
  static const char *	SizeNames[TOTAL_SIZES];
  static const MyString 	m_strSelf;
  int 			m_nCurrentHpts,
  			m_nMaxHpts,
  			m_nFuguePlaneTime;
  short int 		m_nLevel,
  			m_nNoBareHandDamDice,		//How many times to roll
  			m_nTypeBareHandDamDice,	//What did dice to roll
  			m_nPlusBareHandDamDice,	//damage to add to it;
  			m_nAlignment, // good, bad, or ugly
  			m_nMaxManaPts,
  			m_nManaPts,
  			m_nMaxMovePts,
  			m_nMovePts,
  			m_nWorshipedGod,
  			m_nByScreenSize,
  			m_nPracticeSessions, //number of practice session the player has
  			m_nWimpy; //what will they flee at?
  room_vnum 		m_lLastRentRoom,				//duh
  			m_lPlayerSetHomeTown;
  float 		m_fFrags; //how many race war kills
  int 			m_nExperience,
  			m_nPreference, //holds togs for chars and mob flags for mobs
  			m_nSpellSphere,
  			m_nSkillSphere,
  			m_nPosition;	// Character position
  skill_type 		m_Skills[MAX_MUD_SKILLS];
  skill_type 		m_Spells[MAX_SPELLS_FOR_ANY_CLASS];
  skill_type 		m_Languages[TOTAL_LANGUAGES];
  sMoney 		m_CashOnHand;
  sMoney *		m_pBankCash; //So we dont' take up the memory for NPCs
  MyString 		m_strDescription,
  			m_strLongDescription,
  			m_strName,
  			m_strPassword,
  			m_strAlias,
  			m_strTitle,
  			m_strGuildName;
  MyString 		m_strPoofIn,
  			m_strPoofOut;
  CAttributes *		m_pAttributes;
  MyList<CObject *> 	Inventory;
  sAffect 		m_Affects[TOTAL_AFFECTS];
  MyString 		CurrentCommand;
  bool 			SaveToFile(bool bAllowNew);
  char *		SkillWording(skill_type nSkilled);
  bool 			IsPlayerSaved();
  CCharacterAttributes &operator=(sSaveChar &Char);
  void 			CheckFrags(sSaveChar &s);
  void			Reset();
  long			GetAccountNum() { return m_lAccountNum; }
  short			GetSlot() { return m_nSlot; }
  //
  long			m_lAccountNum;
  short			m_nSlot;

private:
  static MyMap <MyString,short> m_SkillNamesMap;
  static MyString 	m_SkillNamesArray[MAX_MUD_SKILLS];
  static bool 		Initialized;
  void 			InitSkillNames();
  void 			InitStatics();
  void 			RemoveDeletedChars();
  void 			BootPlayers();
  CGame *		m_pGame;
};
#endif

