#include "stdinclude.h"
// Database includes
#include <mysql/mysql.h>
#include <libsql++.h>
#include <libwolfshade.h>
//
#include "CRoom.h"
#include "CGame.h"
#include "CObjectManager.h"
#include "CDungeon.h"

//#define DEB(x) x; fflush(stdout);
#define DEB(x)


CDungeon::CDungeon(CGame *pGame,long vnumstart,long xm,long ym,long zm,long seed)
:CGen(pGame,xm,ym)
,m_z(zm)
,m_seed(seed)
,m_tmp(NULL)
,m_startpos(xm,ym,zm)
{
	SetVnumStart(vnumstart);

	m_tmp = new char *[m_z];
	for (int i = 0; i < m_z; i++)
	{
		m_tmp[i] = new char[m_x * m_y];
	}

	for (int i = 0; i < 128; i++)
		v[i] = i;

	v['A'] = ' ';
	v['B'] = ' ';
	v['C'] = 'X';
	v['D'] = ' ';
	v['E'] = 'X';
	v['F'] = ' ';
	v['G'] = 'X';

} // constructor


CDungeon::~CDungeon()
{
	if (m_tmp)
	{
		for (int i = 0; i < m_z; i++)
			delete m_tmp[i];
		delete m_tmp;
	}
} // destructor


void CDungeon::generate()
{
	sPos pos(m_x,m_y,m_z);
	int count = 0;

	srandom(m_seed);
	for (int i = 0; i < m_z; i++)
	{
		pos.z = i;
		for (int j = 0; j < m_y; j++)
		{
			pos.y = j;
			for (int k = 0; k < m_x; k++)
			{
				pos.x = k;
				char c = random() % 8 + 64;
				if (c == 64)
					c = random() % 24 + 'a';
				SetAt(pos,c);
				count++;
			}
		}
	}
} // generate


void CDungeon::display(int z)
{
	sPos pos(m_x,m_y,m_z);
	int i = z;

	ClearVisited();
	{
		pos.z = i;
		printf("Level %d\n",i);
		for (int j = 0; j < m_y; j++)
		{
			pos.y = j;
			printf(" |");
			for (int k = 0; k < m_x; k++)
			{
				pos.x = k;
				if (visited(pos) && GetAt(pos) == ' ')
					printf(".");
				else
					printf("%c",GetAt(pos)); //v[m_tmp[i][j * m_x + k]]);
			}
			printf("|\n");
		}
	}
}


char CDungeon::GetAt(sPos &p)
{
	return v[m_tmp[p.z][p.y * m_x + p.x]];
}


bool CDungeon::connect_levels()
{
	CArea prev_lvl(this);
	CArea this_lvl(this);

	GetLargestArea(0, prev_lvl);
	m_startpos = prev_lvl.GetPosition(random() % prev_lvl.size());
	SetAt(m_startpos,'U');
	for (int z = 1; z < m_z; z++)
	{
		sPos pos(m_x,m_y,m_z);
		pos.z = z - 1;
		GetLargestArea(z, this_lvl);
		if (!connect(prev_lvl, this_lvl, z))
		{
//			fprintf(stderr,"failed to connect levels %d and %d\n",z - 1,z);
			return false;
		}
		//
		prev_lvl = this_lvl;
	}
	return true;
}


void CDungeon::GetLargestArea(int z,CArea &area)
{
	sPos pos(m_x,m_y,m_z);
	areas_t vec;

	ClearVisited();

	pos.z = z;
	for (int y = 0; y < m_y; y++)
	{
		pos.y = y;
		for (int x = 0; x < m_x; x++)
		{
			pos.x = x;
			if (!visited(pos) && GetAt(pos) != 'X')
			{
				CArea *tmp = new CArea(this);
				vec.insert(vec.end(),tmp);
				tmp -> Add(pos); // (recursive)
			}
		}
	}
	size_t size = 0;
	CArea *found = NULL;
	for (areas_t::iterator it = vec.begin(); it != vec.end(); it++)
	{
		CArea *tmp = *it;
		if (tmp -> size() > size)
		{
			found = tmp;
			size = tmp -> size();
		}
	}
	if (found)
		area = *found;
	for (areas_t::iterator it = vec.begin(); it != vec.end(); it++)
	{
		CArea *tmp = *it;
		delete tmp;
	}
	for (int y = 0; y < m_y; y++)
	{
		pos.y = y;
		for (int x = 0; x < m_x; x++)
		{
			pos.x = x;
			if (!area.InList(pos))
				SetAt(pos,'X');
		}
	}
}


bool CDungeon::connect(CArea &area0,CArea &area1,int z)
{
	pos_t common;
	for (pos_t::iterator it = area0.begin(); it != area0.end(); it++)
	{
		sPos pos = *it;
		if (GetAt(pos) != 'U')
		{
			pos.z = z;
			if (area1.InList(pos))
			{
				pos.z = z - 1;
				common.insert(common.end(),pos);
			}
		}
	}
	if (!common.size())
		return false;
	int r = random() % common.size();
	sPos pos = common[r];
	SetAt(pos,'N'); // stairs down
	pos.z = z;
	SetAt(pos,'U');
	return true;
}


void CDungeon::SetAt(sPos &pos,char c)
{
	m_tmp[pos.z][pos.y * m_x + pos.x] = c;
}


long CDungeon::GetVnum(sPos &pos)
{
	return pos.z * m_x * m_y + pos.y * m_x + pos.x + GetVnumStart();
}


long CDungeon::GetVnumEnd()
{
	return GetVnumStart() + m_x * m_y * m_z;
}


void CDungeon::implement(CRoom *room_above)
{
	sPos pos(m_x,m_y,m_z);
	bool north;
	bool south;
	bool east;
	bool west;

	// vroom
	//  20 - cell
	//  21 - east-west
	//  22 - north-south
	//  23 - three-way
	//  24 - four-way
	//  25 - corner

	// item
	//  40 - stairs up
	//  41 - stairs down

	for (int z = 0; z < m_z; z++)
	{
		pos.z = z;
		for (int y = 0; y < m_y; y++)
		{
			pos.y = y;
			for (int x = 0; x < m_x; x++)
			{
				pos.x = x;
				if (GetAt(pos) != 'X')
				{
					sPos pos2(pos);
					pos2.MoveNorth();
					north = (GetAt(pos2) != 'X');
					pos2 = pos;
					pos2.MoveSouth();
					south = (GetAt(pos2) != 'X');
					pos2 = pos;
					pos2.MoveEast();
					east = (GetAt(pos2) != 'X');
					pos2 = pos;
					pos2.MoveWest();
					west = (GetAt(pos2) != 'X');
					int count = 0;
					int rnum = 20;
					if (north)
						count++;
					if (south)
						count++;
					if (east)
						count++;
					if (west)
						count++;
					switch (count)
					{
					case 1:
						rnum = 20;
						break;
					case 2:
						if (east && west)
							rnum = 21;
						else
						if (north && south)
							rnum = 22;
						else
							rnum = 25;
						break;
					case 3:
						rnum = 23;
						break;
					case 4:
						rnum = 24;
						break;
					}
					CRoom *pRoom;
					if ((pRoom = GetVirtualRoom(rnum)) != NULL)
					{
						CRoom *pNewRoom = new CRoom(pRoom);
DEB(printf("Adding room: %s\n",(char *)pNewRoom->GetName());)

						pNewRoom -> m_nXpos = pos.x; // hopefully these are ignored elsewhere
						pNewRoom -> m_nYpos = pos.y;
						pNewRoom -> m_lVnum = GetVnum( pos );
DEB(printf(" Roomvnum = %ld\n",pNewRoom -> m_lVnum);)
						pNewRoom -> m_bZoneRoom = false;
						GetGame()->GetWorld()->GetWorldMap().Add(pNewRoom->m_lVnum,pNewRoom);

						if (pos == m_startpos)
						{
							Connect(room_above,DOWN,pNewRoom);
							Connect(pNewRoom,UP,room_above);
						}
						if (north)
						{
							pos2 = pos;
							pos2.MoveNorth();
							Connect(pNewRoom,NORTH,GetVnum(pos2));
						}
						if (south)
						{
							pos2 = pos;
							pos2.MoveSouth();
							Connect(pNewRoom,SOUTH,GetVnum(pos2));
						}
						if (east)
						{
							pos2 = pos;
							pos2.MoveEast();
							Connect(pNewRoom,EAST,GetVnum(pos2));
						}
						if (west)
						{
							pos2 = pos;
							pos2.MoveWest();
							Connect(pNewRoom,WEST,GetVnum(pos2));
						}
						switch (GetAt(pos))
						{
						case 'U':
							// add item 40
							GetGame()->GetObjectManager()->Create(40,pNewRoom,-1);
							pos2 = pos;
							if (pos2.MoveUp())
							{
								Connect(pNewRoom,UP,GetVnum(pos2));
							}
							break;
						case 'N':
							// add item 41
							GetGame()->GetObjectManager()->Create(41,pNewRoom,-1);
							pos2 = pos;
							pos2.MoveDown();
							Connect(pNewRoom,DOWN,GetVnum(pos2));
							break;
						}
DEB(printf("Adding room: %s\n",(char *)pNewRoom->GetName());)

					}

				} // if != 'X'
			}
		}
	}
}


CGen::sPos CDungeon::GetPosFromVnum(long vnum)
{
	sPos pos(m_x,m_y,m_z);
	pos.x = (vnum - GetVnumStart()) % m_x;
	pos.y = ((vnum - GetVnumStart()) / m_x) % m_y;
	pos.z = (vnum - GetVnumStart()) / (m_x * m_y);
	return pos;
}



