//////////////////////////////////////////////////////////////
//Wolfshade MUD server
//Copyright (C) 1999 Demetrius and John Comes
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////
/*
*
*
*  Implementation of CGame
*
*/
#include "stdinclude.h"
#ifdef PRAGMAHDRSTOP
#pragma hdrstop
#endif
// Database includes
#include <mysql/mysql.h>
#include <libsql++.h>
#include <libwolfshade.h>
//
#include "CNPC.h"
#include "CMageClass.h"
#include "CPsionicistClass.h"
#include "CEmpathClass.h"
#include "CDefilerClass.h"
#include "CRangerClass.h"
#include "CShamanClass.h"
#include "CMindflayerClass.h"
#include "CDruidClass.h"
#include "CPaladinClass.h"
#include "CAntiPaladinClass.h"
#include "CClericClass.h"
#include "main.h"
#include "CZoneManager.h"
#include "MySocket.h"
#include "CRoom.h"
#include "CInterp.h"
#include "CCharIntermediate.h"
#include "CObjectManager.h"
#include "CMobManager.h"
// timers
#include "CResetTimer.h"
#include "CFightTimer.h"
#include "CObjAffectTimer.h"
#include "CResourceTimer.h"
#include "CResourceCharTimer.h"

#include "CGame.h"

//#define DEB(x) x; fflush(stderr);
#define DEB(x)


//////////////////////////////////////////////////////////
//	statics

bool CGame::m_bShutDown = false;
#ifndef NOFIGHTLL
MyList<CCharacter *> CGame::m_FightLL;
#endif // NOFIGHTLL


//////////////////////////////////////////////////
///	Non STatics

CGame::CGame(MyString strOpts) : m_db("localhost","root","","wolfshade")
,m_bUseDB(true)
,m_pMobManager(NULL)
,m_pZoneManager(NULL)
,m_pObjectManager(NULL)
,m_pInterpetor(NULL)
,m_pWorld(NULL)
,m_pConnections(NULL)
,m_bGenerateDB(false)
,m_strZoneIndex(CZoneManager::DEFAULT_ZONEINDEX)
,m_strMobIndex(CMobManager::DEFAULT_MOBINDEX)
,m_strObjIndex(CObjectManager::DEFAULT_OBJINDEX)
,m_strRoomIndex(CWorld::DEFAULT_REALINDEX)
,m_bNoColor(false)
,m_bNoNameServer(false)
{
	int nPos;

	m_bNoNameServer = (strOpts.Find("-n")==-1);
	if (strOpts.Find("-nocolor") != -1)
	{
		m_bNoColor = true;
	}
	if (strOpts.Find("-generatedb") != -1)
	{
#ifdef DODB
		m_bGenerateDB = true;
		m_bUseDB = false;
#else
		MudLog << "Define DODB (in the Makefile) and recompile to use the -generatedb switch" << endl;
#endif
	}
	if (strOpts.Find("-classic") != -1)
	{
		MudLog << "Booting using files..." << endl;
		m_bUseDB = false;
	}
	if ( (nPos = strOpts.Find("-z")) > -1)
	{
		m_strZoneIndex = strOpts.Right(nPos + 1).GetWordAfter(0);
	}
	if ( (nPos = strOpts.Find("-m")) > -1)
	{
		m_strMobIndex = strOpts.Right(nPos + 1).GetWordAfter(0);
	}
	if ( (nPos = strOpts.Find("-o")) > -1)
	{
		m_strObjIndex = strOpts.Right(nPos + 1).GetWordAfter(0);
	}
	if ( (nPos = strOpts.Find("-r")) > -1)
	{
		m_strRoomIndex = strOpts.Right(nPos + 1).GetWordAfter(0);
	}

	// dynamically allocate so that later we can have mini mud ect
	// Zone manager
	m_pZoneManager = new CZoneManager(this, m_strZoneIndex );
	// Mob manager
	m_pMobManager = new CMobManager(this, m_strMobIndex);
	// Object manager
	m_pObjectManager = new CObjectManager(this, m_strObjIndex);
	// World
	m_pWorld = new CWorld(this, m_strRoomIndex);
	//must be after world boots ;)
	m_pZoneManager -> ResetZones(true,*m_pObjectManager,*m_pMobManager);

	// Interpreter
	m_pInterpetor = new CInterp(this);

	m_pObjectManager -> LoadCorpses();

	// Listen socket
	m_pConnections = new MySocket(this, strOpts);

	MudLog << "Size of CCharacter Class: " << sizeof(CCharacter) << endl;

	// Save Races In Database
#ifdef DODB
	if (GenerateDB())
	{
		for (int i = 1; i < TOTAL_RACES; i++)
		{
			CRacialReference *tmp = (CRacialReference *)cGRR.GetRaceReference(i);
			MudLog << "Saving race '" << GVM.GetRaceName(i) << "' in database" << endl;
			tmp -> SaveRace(GetDatabase());
		}
	}
#endif

	// Add timers
	m_timerfac.AddTimer(new CResetTimer(&m_timerfac,m_pZoneManager,m_pMobManager,m_pObjectManager));
	m_timerfac.AddTimer(new CObjAffectTimer(&m_timerfac,m_pObjectManager));
	m_timerfac.AddTimer(new CResourceTimer(&m_timerfac,this));
}


CGame::~CGame()
{
DEB(printf("~CGame()\n");)
	m_timerfac.DeleteTimers();

DEB(printf("~CGame() 1\n");)
	delete m_pConnections;
DEB(printf("~CGame() 2\n");)
	delete m_pInterpetor;
DEB(printf("~CGame() 3\n");)
	delete m_pObjectManager;
DEB(printf("~CGame() 4\n");)
	delete m_pMobManager;
DEB(printf("~CGame() 5\n");)
	delete m_pZoneManager;
	//must be last
DEB(printf("~CGame() 6\n");)
	delete m_pWorld;

DEB(printf("~CGame() 7\n");)
	CMage::CleanNewedStaticMemory();
	CPsionicist::CleanNewedStaticMemory();
	CEmpath::CleanNewedStaticMemory();
	CCleric::CleanNewedStaticMemory();
	CDefiler::CleanNewedStaticMemory();
	CRanger::CleanNewedStaticMemory();
	CMindflayer::CleanNewedStaticMemory();
	CShaman::CleanNewedStaticMemory();
	CDruid::CleanNewedStaticMemory();
	CPaladin::CleanNewedStaticMemory();
	CAntiPaladin::CleanNewedStaticMemory();
DEB(printf("~CGame()...\n");)
}


//this is it the main guy the big cheese

void CGame::Heartbeat(CMudTime &MudTime)
{
//	int lPulse = 0;
	long lPulse = 0;

	while(!m_bShutDown)
	{
//		lPulse = CMudTime::GetCurrentPulse();
		lPulse++;

		m_timerfac.Pulse( lPulse );

		//we get commands every pulse
#ifndef SOCKETSTHREAD
		m_pConnections -> GetCommandsFromSockets();
#endif

		// socket ==> Descriptors<sPlayerDescriptor>, NotCharsYet<CCharIntermediate>
#ifndef LISTENTHREAD
		m_pConnections -> NewConnections();  //return bool maybe use throws instead
#endif // LISTENTHREAD

		// CharQ<CCharIntermediate> ==> GVM.Add() to PlayerLL<CCharacter>
		if (m_pConnections -> GetCharQ().GetCount())
		{
			GetCharactersFromQ(m_pConnections -> GetCharQ());
		}

		// NotCharsYet ==> CharQ<CCharIntermediate>
		m_pConnections -> DoNotCharsYet();

		DoCharacterCommands(lPulse);

#ifndef NOFIGHTLL
		if (!(lPulse%CMudTime::PULSES_PER_DO_VIOLENCE))
		{
			DoViolence();
		}
#endif // NOFIGHTLL
/*
		// Replaced by CResetTimer
		if (!(lPulse%CMudTime::PULSES_PER_CHECK_ZONE_RESET))
		{
			m_pZoneManager -> ResetZones(false,*m_pObjectManager,*m_pMobManager);
		}
*/
/*
		if (!(lPulse%CMudTime::REDUCE_OBJECT_TIMERS))
		{
			m_pObjectManager -> ReduceObjectAffects(CMudTime::REDUCE_OBJECT_TIMERS);
		}
*/
		//
		MudTime.NextPulse();
	}
	DoNiceShutdown();
}


//gets the characters from the MySocket Q and puts them in the game.

void CGame::GetCharactersFromQ(CLinkList<CCharIntermediate> &CharQ)
{
	POSITION pos = CharQ.GetStartPosition();

	// CharQ == MySocket::CharQ<CCharIntermediate *>

	if (!pos)
		return;

	CCharIntermediate *CharFromQ = NULL;
	CharFromQ = CharQ.GetNext(pos);
fprintf(stderr,">> Getting CCharIntermediate From MySocket::CharQ\n");
fflush(stderr);
	//Do we want to do this if there's 20 ppl in Q all 20 ppl must
	//execute before we move to next step.  Or do we just want to
	//Do One person?
	//ONE!!!!!!
	CCharacter *CharToAdd;
	CharToAdd = CalcClassPtr(CharFromQ);
//	long lPos = 0;
	if (CharToAdd -> IsNewCharacter())
	{
fprintf(stderr," * Char from CharQ IsNewCharacter()\n");
fflush(stderr);
		if (CharToAdd -> ShouldBeImp())
		{
			for(register int i=0;i<LVL_IMP;i++)
			{
				CharToAdd -> AdvanceLevel(true,false);
			}
			CharToAdd -> SendToChar("You will be imp.\r\n");
		}
		else
		{
			CharToAdd -> AdvanceLevel(true,false);
		}
		//Add player to player file pos
fprintf(stderr," * Saving new character\n");
fflush(stderr);
		CharToAdd -> Save(false);
		MudLog << "New player : ";
	}
	else
	{
fprintf(stderr," * Char from CharQ is not a new character\n");
fflush(stderr);
		if (CharToAdd -> IsGod())
		{
			GVM.AddGod(CharToAdd);
		}
	}
	MyString strGodMsg;
	MudLog << CharToAdd -> GetName() << " has entered the game" << endl(strGodMsg);
	GVM.SendToGods(strGodMsg,CCharacterAttributes::PREFERS_GOD_CONNECTION_LOG,NULL);

	//Add to gobal LL after we advance level
	GVM.Add(CharToAdd);

	CharToAdd -> GetRoom() -> SendToRoom("%s enters the game.\r\n",CharToAdd);
	CharToAdd -> GetRoom() -> Add(CharToAdd);
	CharToAdd -> DoLook();
	CharToAdd -> SendPrompt();
	CharToAdd -> CheckAge();

	CharQ.Remove(CharFromQ,pos);
}


//checks to see if character has a command if so sends it to the
//interpetor.

void CGame::DoCharacterCommands(const int lPulse)
{
	CCharacter *ch = NULL;
	POSITION pos = GVM.PlayerLL.GetStartPosition();
	while(pos)
	{
		ch = GVM.PlayerLL.GetNext(pos);

		//moved the regeneration in here
		// so we don't have to run thought the LL again...
		ch -> Regenerate();

		if (!ch -> IsNPC())
		{
			sPlayerDescriptor *pDesc = (sPlayerDescriptor *)ch -> GetDescriptor();
			if (pDesc -> m_bDirty && pDesc -> IsPlaying() )
			{
				ch -> SendPrompt();
				pDesc -> m_bDirty = false;
			}
		}
		if (!(lPulse%CMudTime::PULSES_PER_CHECK_REMOVE_STUN))
		{
			if (ch -> InPosition(POS_STUNNED) && ch -> StatCheck(CON))
			{
				ch -> RemovePosition(POS_STUNNED);
			}
		}
		if (!(lPulse%CMudTime::PULSES_PER_CHECK_REMOVE_KOED))
		{
			if (ch -> InPosition(POS_KOED) && ch -> StatCheck(CON))
			{
				ch -> RemovePosition(POS_KOED);
			}
		}
		Autosave(ch,true);
		ch -> CheckAge();

		///////////////////////////////
		//	do track
		//  For NPC's this will also move mob (as in regular mob movement)
		//	if it's time to
		ch -> Track();

		///////////////////////////////
		//check if it is void time
		// if they have a command m_lVoidTime is set to 0 is HasCommand fnc
		//	each time CheckVoid is fun m_lVoidTime is incremented
		//  check void returns false for all NPC's
		if (ch -> CheckVoid())
		{
			ch -> Save();
			if (!ch -> IsLinkDead())
			{
				ch -> SendToChar("Voiding ....");
				m_pConnections -> AddCharFromDeath(ch);
			}
			ch -> ReleaseMemoryForObjects();
			ch -> GetRoom() -> Remove(ch);
			GVM.Remove(ch,pos);
		}
		else
		if (ch -> ShouldCutLink())
		{
			m_pConnections -> KillSocket(ch -> KillSocket());
		}
		//rent or camp command
		else
		if (ch -> ShouldRent())
		{
			if (ch -> IsNPC())
			{
				ErrorLog << "Trying to rent an NPC!\r\n" << endl;
			}
			else
			{
				ch -> Save();
				m_pConnections -> AddCharFromDeath(ch);
				ch -> ReleaseMemoryForObjects();
				ch -> GetRoom() -> Remove(ch);
				GVM.Remove(ch,pos);
			}
		}
		else
		if (ch -> IsDead() && ch -> IsAffectedBy(CCharacter::AFFECT_AGE_DEATH))
		{
			//uncomment the 2 lines below and you will have
			//age death with no corpse and eq left after words
			// \todo  DISCUSS!
			//StopAllFromFighting(ch);
			KillChar(ch);
			ch -> DeleteChar();
			ch -> RemoveAffect(CCharacter::AFFECT_AGE_DEATH);
			//ch -> ReleaseMemoryForObjects();
			m_pConnections -> KillSocket(ch -> KillSocket());
			GVM.Remove(ch,pos);
		}
		else
		if (ch -> IsDead())
		{
			KillChar(ch);
			if (!ch -> IsNPC() && !ch -> IsLinkDead())
			{
				m_pConnections -> AddCharFromDeath(ch);
			}
#ifdef NOFIGHTLL
			for (timers_t::iterator it = m_timerfac.GetList().begin();
				it != m_timerfac.GetList().end(); it++)
			{
				CFightTimer *tmp = dynamic_cast<CFightTimer *>(*it);
				if (tmp && !tmp -> KillThis() )
				{
					if (tmp -> GetAttacker() == ch)
					{
						ErrorLog << "removing " << ch -> GetName() << " after death!" << endl;
						tmp -> KillTimer();
					}
				}
			}
#else
			if (m_FightLL.DoesContain(ch))
			{
				ErrorLog << "removing " << ch -> GetName() << " after death!" << endl;
				m_FightLL.Remove(ch);
			}
#endif
			GVM.Remove(ch,pos);
		}
		else
		if (ch -> HasCommand())
		{
			if (!ch -> IsNPC())
			{
				sPlayerDescriptor *pDesc = (sPlayerDescriptor *)ch -> GetDescriptor();
				pDesc -> m_bDirty = true;
			}
			m_pInterpetor -> InterpretCommand(ch);
		}
		//we do NPC stuff only if they don't already have
		//a command
		else
		if (ch -> IsNPC())
		{
			//only do ai if they are in correct position
			if ((!(lPulse % CNPC::DO_AI)) &&
				!ch -> InPosition(POS_SLEEPING) &&
				!ch -> InPosition(POS_KOED) &&
				!ch -> InPosition(POS_INCAP) )
			{
				ch -> DoAI(m_pInterpetor);
			}
		}
	}
}


////////////////////////////////////
//	DoViolence()
//	Ok here we do some killin
//	cycle through fight link list
//	see if person you are fighting is fighting
//	if not make him fight you (add to LL)
//	next ATTACK
//  If you are dead we remove you from fight LL
//	and kill you we do the same to your target if they die
//	if either can't fight then we assertfighting NULL
//	and remove them you or your target from the LL
//	then we run though and send prompt to all
//	if they aren't fighting remove them from LL
//	written by: Demetrius Comes
//	5/13/98
////////////////////////////////////
#ifndef NOFIGHTLL
void CGame::DoViolence()
{
	POSITION pos = m_FightLL.GetStartPosition();
	CCharacter *pCh;
	CCharacter *pDefender;

	while(pos)
	{
		pCh = m_FightLL.GetNext(pos);

		//are we fighting and can we fight
		if ( (pDefender = pCh -> IsFighting()) != NULL &&
			pCh -> CanFight() &&
			!pDefender -> IsDead() &&
			pCh -> IsInRoom(pDefender) )
		{
			//if are defender isn't fighting
			//and not in the fightLL make him fight
			if (!pDefender -> IsFighting() &&
				pDefender -> CanFight() &&
				!m_FightLL.DoesContain(pDefender))
			{
				m_FightLL.Add(pDefender);
				pDefender -> AssertFighting(pCh);
			}

			//Add memory for NPC's and track target
			pCh -> AddMemory(pDefender);

			//attack target
			pCh -> Attack();

			//check for autoflee
			pDefender -> DoAutoFlee();

			//if you can't fight stop fighting
			if (!pCh -> CanFight() || 
				!pCh -> IsFighting() ||
				!pCh -> IsInRoom(pDefender))
			{
				pCh -> AssertFighting(NULL);
				//must remove them from LL other wise
				//KillChar calls stopallfromfighting
				//and that will remove them from LL with
				//OUT having the position variable!
				m_FightLL.Remove(pCh,pos);
				//if your dead from you from LL and kill you
				//KillChar set all characters fighting you to
				//assertfighting(NULL)
				if (pCh -> IsDead())
				{
					KillChar(pCh);
				}
			}

			//if your defender can't fight stop them from fighting
			if (!pDefender -> CanFight() ||
				!pDefender -> IsFighting() ||
				!pCh -> IsInRoom(pDefender))
			{
				pDefender -> AssertFighting(NULL);
				//must remove them from LL other wise
				//KillChar calls stopallfromfighting
				//and that will remove them from LL with
				//OUT having the position variable!
				m_FightLL.Remove(pDefender,pos);
				if (pDefender -> IsDead())
				{
					KillChar(pDefender);
				}
			}
		}
		//your not fighting anyone or the person your
		//fighting has left the room.
		else
		{
			pCh -> AssertFighting(NULL);
			m_FightLL.Remove(pCh,pos);
		}
	} // while (pos)
/*
	pos = m_FightLL.GetStartPosition();
	while(pos)
	{
		pCh = m_FightLL.GetNext(pos);
		pCh -> SendPrompt();
	}
*/
}
#endif // NOFIGHTLL


//////////////////////////////////////
//	Kill Character
//	Remove from Room LL
//	Remove from Game LL
//	save char
//	Make a CCharIntermediate setting state to menu
//	Add to MySocket::NotCharYet
//	written by: Demetrius Comes
//	5/19/98
//////////////////////////////////////

void CGame::KillChar(CCharacter * pDeadChar)
{
	DEB(MyString tmp = pDeadChar -> GetName();
	tmp.Colorize(false);
	fprintf(stderr,"KillChar('%s')\n",(char *)tmp);)

	//if player is not in a room then thay have already been killed
	if (pDeadChar -> GetRoom() -> DoesContain(pDeadChar))
	{
		StopAllFromFighting(pDeadChar);
//		long lPos = 0;

		//send death message
		pDeadChar -> GetRoom() -> DeathCry(pDeadChar);

		//now remove from room
		pDeadChar -> GetRoom() -> Remove(pDeadChar);
		pDeadChar -> UpDatePosition(POS_DEAD,false);

		//put save here instead of in DoCharacterCommands
		//that way if we crash after they died there is no
		//chance of the death not saving
		pDeadChar -> SetFuguePlane();

		//add the corpse to the room
		if (pDeadChar -> ShouldMakeCorpse())
		{
			CObject *pObj = new CCorpse(pDeadChar);
			pDeadChar -> GetRoom() -> Add(pObj, false);
			CObjectManager::Add(pObj);
		}
		if (!pDeadChar -> IsNPC())
		{
			pDeadChar -> Save(true);
		}
		else
		{
			m_pMobManager -> ReduceNumberInGame(pDeadChar -> GetVnum());
		}
	}
	DEB(fprintf(stderr,"KillChar() ok\n");)
} // KillChar


////////////////////////////////
//	Autosave
//	Saves all players currently
//	connnected.
//	written by: Demetrius Comes
//	7/2/98
/////////////////////////////

void CGame::Autosave(CCharacter *pSaveChar, bool bSendMsg)
{
	if (!pSaveChar -> IsNPC() &&
		(!(CMudTime::GetCurrentPulse()%CMudTime::PULSES_PER_DO_AUTO_SAVE)))
	{
		pSaveChar -> Save(false);
		if (bSendMsg)
		{
			if (pSaveChar -> GetName() == (MyString)"Aahrguz")
			{
				pSaveChar -> SendToChar("Autosaving... oh but wait, AAHRGUZ doesn't like autosave!\r\n");
			}
			else
			{
				pSaveChar -> SendToChar("Autosaving...\r\n");
			}
		}
	}
}


//////////////////////////
//	Remove all persons fighting pCH
/////////////////////////

void CGame::StopAllFromFighting(CCharacter * pCh)
{
	POSITION pos = GVM.PlayerLL.GetStartPosition();
	CCharacter *pCurrent;
	
	DEB(MyString tmp = pCh -> GetName();
	tmp.Colorize(false);
	fprintf(stderr,"StopAllFromFighting('%s')\n",(char *)tmp);)

	while(pos)
	{
		pCurrent = GVM.PlayerLL.GetNext(pos);
#ifdef NOFIGHTLL
		for (timers_t::iterator it = m_timerfac.GetList().begin();
			it != m_timerfac.GetList().end(); it++)
		{
			CFightTimer *tmp = dynamic_cast<CFightTimer *>(*it);
			if (tmp && !tmp -> KillThis() )
			{
				if (tmp -> GetAttacker() == pCh)
				{
					tmp -> KillTimer();
					pCh -> AssertFighting(NULL);
				}
				if (tmp -> GetDefender() == pCh)
				{
					tmp -> GetAttacker() -> AssertFighting(NULL);
					tmp -> KillTimer();
				}
			}
		}
#else
		if (pCurrent==pCh)
		{
			m_FightLL.Remove(pCh);
			pCh -> AssertFighting(NULL);
		}
		if (pCurrent -> IsFighting()==pCh)
		{
			pCurrent -> AssertFighting(NULL);
			m_FightLL.Remove(pCh);
		}
#endif
		if (pCh -> IsDead())
		{
			pCurrent -> RemoveFromMemory(pCh);
		}
	}
//	DEB(fprintf(stderr,"StopAllFromFighting() ok\n");)
} // StopAllFromFighting


/////////////////////////////////////////
//	Static:
//	Add a fighter to the LL
#ifndef NOFIGHTLL
void CGame::AddFighter(CCharacter *pFighter)
{
	if (m_FightLL.DoesContain(pFighter))
	{
//		ErrorLog << "Trying to add " << pFighter -> GetName() << " twice to fight LL." << endl;
		return;
	}
	m_FightLL.Add(pFighter);
}
#else
//	Not a static anymore
void CGame::AddFighter(CCharacter *pFighter,CCharacter *pTarget)
{
	DEB(MyString tmp1 = pFighter -> GetName();
	MyString tmp2 = pTarget -> GetName();
	tmp1.Colorize(false);
	tmp2.Colorize(false);
	fprintf(stderr,"AddFighter('%s','%s')\n",(char *)tmp1,(char *)tmp2);)

	for (timers_t::iterator it = m_timerfac.GetList().begin(); it != m_timerfac.GetList().end(); it++)
	{
		CFightTimer *tmp = dynamic_cast<CFightTimer *>(*it);
		if (tmp && !tmp -> KillThis() )
		{
			if (tmp -> GetAttacker() == pFighter)
			{
//				ErrorLog << "Trying to add " << pFighter -> GetName() << " twice to fight LL." << endl;
				return;
			}
		}
	}
	long pulses = 2 * CMudTime::PULSES_PER_REAL_SECOND;
	if (pFighter -> IsNPC())
	{
		pulses += 2;
	}
	else
	{
		pulses -= 2;
	}
	m_timerfac.AddTimer(new CFightTimer(this,&m_timerfac,m_pMobManager,pFighter,pTarget,pulses));
	DEB(fprintf(stderr,"AddFighter() ok\n");)
} // AddFighter
#endif // NOFIGHTLL


//////////////////////////////////////
//	Save everyone before shutting down!

void CGame::DoNiceShutdown()
{
	POSITION pos = GVM.PlayerLL.GetStartPosition();
	CCharacter *pCh;
	while(pos)
	{
		pCh = GVM.PlayerLL.GetNext(pos);
		pCh -> Save();
		pCh -> SendToChar("Game is shutting down...\r\n");
	}
}


Database *CGame::GetDatabase()
{
	return &m_db;
}


MyString CGame::ShowTimers()
{
	MyString strInfo("Type                     Ticks  Time  Active      Keep\r\n");

	strInfo += "------------------------------------------------------\r\n";
	for (timers_t::iterator it = m_timerfac.GetList().begin();
		it != m_timerfac.GetList().end(); it++)
	{
		CTimer *tmp = static_cast<CTimer *>(*it);
		if (tmp)
		{
			char slask[200];
			sprintf(slask,"%24s %5ld %5ld  %s  %s  %s",
				(char *)tmp -> GetType(),
				tmp -> GetTicks(),
				tmp -> GetTime(),
				tmp -> IsActive() ? "ACTIVE    " : "NOT ACTIVE",
				tmp -> KillThis() ? "KILL" : "KEEP",
				(char *)tmp -> GetExtra() );
			strcat(slask,"\r\n");
			strInfo += slask;
		}
		else
			strInfo.Format("%s<missing>\r\n",
				(char *)strInfo);
	}
	return strInfo;
}


void CGame::AddResourceRoom(CRoom *p)
{
	m_ResourceRoom.insert(m_ResourceRoom.end(),p);
}


void CGame::RemoveResourceRoom(CRoom *p)
{
	for (rooms_t::iterator it = m_ResourceRoom.begin();
		it != m_ResourceRoom.end(); it++)
	{
		if (*it == p)
			m_ResourceRoom.erase( it );
	}
}


void CGame::StartResourceTimer(CCharacter *pCh,int skill)
{
	m_timerfac.AddTimer(new CResourceCharTimer(&m_timerfac,2*CMudTime::PULSES_PER_REAL_SECOND,pCh,skill));
}


