//////////////////////////////////////////////////////////////
//Wolfshade MUD server
//Copyright (C) 1999 Demetrius and John Comes
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////
/*
* Game Class - The Main Class
	This class manages the games.  
	The heartbeat function is the game loop.
*
*/

#ifndef _GAME_H_
#define _GAME_H_


class MySocket;
class CRoom;
class CZoneManager;
typedef std::vector<CRoom *> rooms_t;

class CGame
{
  friend class 		CInterp;
  friend void 		CCharacter::DoOtherState();
  friend void 		MySignal(int x);

public:
  			CGame( MyString strOpts );
  			~CGame();

  CTimerFactory *	GetFactory() { return &m_timerfac; }

  static bool 		IsShuttingDown(){return m_bShutDown;}
  static void 		AddFighter(CCharacter *pFighter);
  void			AddFighter(CCharacter *,CCharacter *);
  //
  void 			Heartbeat(CMudTime &MudTime);
  Database *            GetDatabase();
  bool                  UseDB() { return m_bUseDB; }
  CMobManager *		GetMobManager() { return m_pMobManager; }
  CObjectManager *	GetObjectManager() { return m_pObjectManager; }
  CZoneManager *	GetZoneManager() { return m_pZoneManager; }
  CWorld *		GetWorld() { return m_pWorld; }
  bool			GenerateDB() { return m_bGenerateDB; }
  MyString 		ShowTimers();
  bool			NoColor() { return m_bNoColor; }
  bool			NoNameServer() { return m_bNoNameServer; }
  void			AddResourceRoom(CRoom *);
  void			RemoveResourceRoom(CRoom *);
  rooms_t &		GetResourceRooms() { return m_ResourceRoom; }
  void			StartResourceTimer(CCharacter *,int skill);

protected:
  static void 		ShutDownWolfshade() {m_bShutDown = true;}
  //
public: // used by CFightTimer
  void 			StopAllFromFighting(CCharacter *pCh);
protected:
  void 			Autosave(CCharacter *pSaveChar,bool bSendMsg);
  void 			KillChar(CCharacter *pDeadChar);
  void 			DoViolence();

private:
  void 			DoNiceShutdown();
  void 			DoCharacterCommands(const int Pulse);
  void 			GetCharactersFromQ(CLinkList<CCharIntermediate> &NewChars);
  //
  static bool 		m_bShutDown;
#ifndef NOFIGHTLL
  static MyList<CCharacter *> m_FightLL;
#endif
  //
  Database              m_db;
  bool                  m_bUseDB;
  CMobManager *		m_pMobManager;
  CZoneManager *	m_pZoneManager;
  CObjectManager *	m_pObjectManager;
  CInterp *		m_pInterpetor;
  CWorld *		m_pWorld;
  MySocket *		m_pConnections;
  bool			m_bGenerateDB;
  MyString		m_strZoneIndex;
  MyString		m_strMobIndex;
  MyString		m_strObjIndex;
  MyString		m_strRoomIndex; // realroom
  CTimerFactory		m_timerfac;
  bool			m_bNoColor;
  bool			m_bNoNameServer;
  rooms_t		m_ResourceRoom;
};
#endif

