/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "stdinclude.h"
#ifdef PRAGMAHDRSTOP
#pragma hdrstop
#endif
// Database includes
#include <mysql/mysql.h>
#include <libsql++.h>
#include <libwolfshade.h>
//
#ifdef _WOLFSHADE_WINDOWS_
typedef int socklen_t;
#else
#include <sys/socket.h>  //socket functions
#include <sys/resource.h> //timeval stuct declaration
#include <errno.h> //for the type error_t
#include <fcntl.h>  //function header to nonblock a socket
#include <unistd.h> //int close()
#define closesocket close
#include <netdb.h> //hostent declaration
#include <signal.h>
#include <arpa/inet.h>
#define INVALID_SOCKET -1
#endif

#include "MySocket.h"
#include <string.h>
#include <signal.h>
#include "CWolfshadeLog.h"
#include "main.h"
#include "CCharIntermediate.h"
#include "CGame.h"
#include "CMutex.h"
#ifdef SOCKETSTHREAD
#include "MySocketsThread.h"
#endif

#include "CListenThread.h"

//#define DEB(x) x; fflush(stdout);
#define DEB(x)


CListenThread::CListenThread(CGame *pGame,MySocket *pSocket,CMutex *pMutex,int maxplayers,int port) 
:CThread(false)
,m_pMutex(pMutex)
,m_nPort(port)
,m_pGame(pGame)
,m_MaxPlayers(maxplayers)
,m_pConnections(pSocket)
{
	struct sockaddr_in sa;
	MyString Error;

	//make listening socket
	if((ListenSocket = socket(PF_INET,SOCK_STREAM,0)) == INVALID_SOCKET)
	{
#ifdef _WOLFSHADE_WINDOWS_
		Error.Format("%s %d","Error opening network socket ", WSAGetLastError());
#else
		Error.Format("%s %d","Error opening network socket ", errno);
#endif
		CError Err(CRITICAL_ERROR, Error);
		throw &Err;
	}

#if defined(SO_SNDBUF)
	int opt = MAX_COMMAND_BUFFER;
	if(setsockopt(ListenSocket,SOL_SOCKET, SO_SNDBUF, (char *) &opt, sizeof(opt)) < 0)
	{
		CError Err(CRITICAL_ERROR,"Error setting SO_SNDBUF");
		throw &Err;
	}
#endif

#if defined (SO_REUSEADDR)
	opt = 1;
	if (setsockopt(ListenSocket, SOL_SOCKET, SO_REUSEADDR, (char *) &opt, sizeof(opt)) < 0)
	{
		CError Err(CRITICAL_ERROR,"Error setting SO_REUSEADDR\n");
		throw &Err;
	}
#endif

#if defined (SO_LINGER)
	ld.l_onoff = 0;
	ld.l_linger = 0;
	if (setsockopt(ListenSocket,SOL_SOCKET,SO_LINGER,(char *) &ld, sizeof(ld)) < 0)
	{
		ErrorLog << "Informational: SO_LINGER won't set" << endl;
	}
#endif
		
	sa.sin_family = AF_INET;
	sa.sin_port = htons(m_nPort);
	sa.sin_addr.s_addr = htonl(INADDR_ANY);

	if(bind(ListenSocket,(struct sockaddr *) &sa, sizeof(sa)) < 0)
	{
#ifdef _WOLFSHADE_WINDOWS_
		Error.Format("%s %d %s","Error binding network socket: ",
			WSAGetLastError(),"closing socket");
#else
		Error.Format("%s %d %s","Error binding network socket: ",
			errno,"closing socket");
#endif
		shutdown(ListenSocket, SHUT_RDWR);
		closesocket(ListenSocket);
		CError Err(CRITICAL_ERROR,Error);
		throw &Err;
	}
	MudLog << "Wolfshade mud is running on port: " << m_nPort << endl;
	if(NonBlock(ListenSocket)!=0)
	{
#ifdef _WOLFSHADE_WINDOWS_
		Error.Format("%s %d %s","Error in blocking Network socket: ",
			WSAGetLastError(),"\nclosing socket!");
#else
		Error.Format("%s %d %s","Error in blocking network socket: ",
			errno, "\r\nclosing socket!");
#endif
		shutdown(ListenSocket, SHUT_RDWR);
		closesocket(ListenSocket);
		CError Err(CRITICAL_ERROR,Error);
		throw &Err;
	}
	if(listen(ListenSocket,5)!=0)
	{
#ifdef _WOLFSHADE_WINDOWS_
		Error.Format("%s %d %s","Error in making network socket listening sock: ",
			WSAGetLastError(),"\nclosing socket!");
#else
		Error.Format("%s %d %s","Error in making network socket listening socket: ",
			errno,"\r\nclosing socket!");
#endif
		shutdown(ListenSocket, SHUT_RDWR);
		closesocket(ListenSocket);
		CError Err(CRITICAL_ERROR,Error);
		throw &Err;
	}

	// everything ok, release thread
	SetRelease(true);
}


CListenThread::~CListenThread()
{
DEB(printf("~CListenThread()\n");)
	shutdown(ListenSocket, SHUT_RDWR);
	closesocket(ListenSocket);
}


void CListenThread::Run()
{
	fd_set rfds;
	int n;
	struct timeval tv;

	while (!Quit())
	{
		FD_ZERO(&rfds);
		FD_SET(ListenSocket,&rfds);
		tv.tv_sec = 0;
		tv.tv_usec = 500000L;
		n = select(ListenSocket + 1,&rfds,NULL,NULL,&tv);
		if (n < 0)
		{
			// err
		}
		else
		if (!n)
		{
			// timeout
		}
		else
		if (FD_ISSET(ListenSocket,&rfds))
		{
			NewDescriptor();
		}
	}
	SetRunning(false);
}


//checks for new connects

bool CListenThread::NewConnections()
{
/*
	struct timeval NullTime;
	NullTime.tv_sec=0;
	NullTime.tv_usec = 0;
	sPlayerDescriptor *currentDesc;
	POSITION pos = m_pConnections->GetDescriptors().GetStartPosition();
	fd_set input_set;
	fd_set output_set;
	fd_set exception_set;

	//zero input set
	FD_ZERO(&input_set);
	//put listensocket into input set
	FD_SET(ListenSocket,&input_set);
	if(pos==NULL)
	{
		//poll with no wait
		if(select(ListenSocket+1,&input_set,(fd_set *) 0,(fd_set *) 0,NULL)<0)
		{
			ErrorLog << "select < 0\n" << endl;
		}
		else 
		{
			MudLog << "New connection waking up!\r\n";
		}
	}
	FD_ZERO(&input_set);
	FD_ZERO(&output_set);
	FD_ZERO(&exception_set);
	FD_SET(ListenSocket, &input_set);
	while(pos)
	{
		currentDesc = m_pConnections->GetDescriptors().GetNext(pos);
		FD_SET(currentDesc->m_ConnectedSocket,&input_set);
		FD_SET(currentDesc->m_ConnectedSocket,&output_set);
		FD_SET(currentDesc->m_ConnectedSocket,&exception_set);
	}
	//Circle sleeps here before next pulse
	if (select(ListenSocket + 1, &input_set, &output_set, 
						&exception_set, &NullTime) < 0) 
	{
		//ErrorLog << "Select poll";
		//NO NEW CONNECTIONS
		return false;
	}
	// If there are new connections waiting, accept them. 
	if (FD_ISSET(ListenSocket, &input_set))
	{
		NewDescriptor();
	}
	// kick out the freaky folks in the exception set 
	KickOutExceptionSet();
*/
	return true;
}


//handles a new connection

void CListenThread::NewDescriptor()
{
	SOCKET desc;
	struct sockaddr_in peer;
	struct hostent *from;
//	static const MyString buf("Welcome to Wolfshade MUD! Ascii Tile Screen? [y]: ");
	static const MyString strDeny("Sorry too many connections right now.  Try back later.\r\n");

	socklen_t i = sizeof(peer);
	if((desc=accept(ListenSocket,(struct sockaddr *) &peer,&i))==INVALID_SOCKET)
	{
		ErrorLog << "error on accept!\n";
		return;
	}
	NonBlock(desc);

// CListenThread
//  .m_pConnections->Descriptors - connected sockets (sPlayerDescriptor)
//
//  .m_pConnections->NotCharsYet - CCharIntermediate <ej inloggad>
//

	m_pMutex -> Lock();
	int count = m_pConnections->GetDescriptors().GetCount();
	m_pMutex -> Unlock();
	if(count > m_MaxPlayers)
	{
		MyString tmp = strDeny;
		send(desc,tmp/*strDeny*/,strDeny.GetLength(),0);
	}
	else
	{
		sPlayerDescriptor *newbieDesc = m_pConnections -> AddSocket( desc );
/*
		sPlayerDescriptor *newbieDesc = new sPlayerDescriptor(desc);

		//add new socket to descriptor link list
		m_pMutex -> Lock();
		m_pConnections->GetDescriptors().Add(newbieDesc); //(newbieDesc = new sPlayerDescriptor(desc)));
		m_pMutex -> Unlock();

		CCharIntermediate *newbieChar = new CCharIntermediate(m_pGame,newbieDesc);

		//use new descriptor to make character intermediate
		//add this intermediate to link list
		m_pMutex -> Lock();
		m_pConnections->GetNotCharsYet().Add(newbieChar); //(newbieChar = new CCharIntermediate(m_pGame,newbieDesc)));
		m_pMutex -> Unlock();

		//send first screen
		newbieChar->SendToChar(buf);
*/

		//set the socket number
//		MudLog << "The desc is: " << desc << endl;

		//We'll want to get log in time here!
		if(m_pGame->NoNameServer() || !(from = gethostbyaddr((char *) &peer.sin_addr, sizeof(peer.sin_addr), AF_INET)))
		{
			newbieDesc->Host = inet_ntoa(peer.sin_addr);
		}
		else
		{
			newbieDesc->Host = from->h_name;
		}
		{
			MyString strGod;
			m_pMutex -> Lock();
			int count = m_pConnections->GetDescriptors().GetCount();
			m_pMutex -> Unlock();
			MudLog << newbieDesc->Host << " and there are " << count << " connected sockets" << endl(strGod);
			GVM.SendToGods(strGod,CCharacterAttributes::PREFERS_GOD_CONNECTION_LOG,NULL);
		}
	}
}


void CListenThread::KickOutExceptionSet()
{
/*
	sPlayerDescriptor *current = NULL;
	POSITION pos = m_pConnections->GetDescriptors().GetStartPosition();
	fd_set input_set;
	fd_set output_set;
	fd_set exception_set;

	while(pos)
	{
		current = m_pConnections->GetDescriptors().GetNext(pos);
		if(FD_ISSET(current->m_ConnectedSocket,&exception_set))
		{
			ErrorLog << "Exception found: " << current->m_ConnectedSocket <<endl;
			FD_CLR(current->m_ConnectedSocket,&input_set);
			FD_CLR(current->m_ConnectedSocket,&output_set);
			shutdown(current->m_ConnectedSocket, SHUT_RDWR);
			closesocket(current->m_ConnectedSocket);
		}
	}
*/
}


int CListenThread::NonBlock(SOCKET s)
{
	int nRetVal;
#ifdef _WOLFSHADE_WINDOWS_
	unsigned long val;
	val = 1;
	nRetVal = ioctlsocket(s,FIONBIO,&val);
#else
#ifndef O_NONBLOCK
#define O_NONBLOCK O_NDELAY
#endif

	int flags;
	flags = fcntl(s, F_GETFL, 0);
	flags |= O_NONBLOCK;
	if((nRetVal = fcntl(s, F_SETFL, flags)) < 0) 
	{
		MyString str;
		str.Format("Could not non block socket %d",(int)s);
		CError Err(CRITICAL_ERROR,str);
		throw &Err;
	}
#endif
	return nRetVal;
}



