//////////////////////////////////////////////////////////////
//Wolfshade MUD server
//Copyright (C) 1999 Demetrius and John Comes
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////
/**********************************************
*
*  CObjectPrototype Implementation
*
*   DAC 8/4/97
*
************************************************/

#include "stdinclude.h"
#ifdef PRAGMAHDRSTOP
#pragma hdrstop
#endif
// Database includes
#include <mysql/mysql.h>
#include <libsql++.h>
#include <libwolfshade.h>
//
#ifdef _WOLFSHADE_WINDOWS_
//#include <strstrea.h>
#else
//#include <strstream.h>
#endif
#include "CObjectPrototype.h"
#include "CWolfshadeLog.h"
#include "CPortal.h"
#include "CRoom.h"
#include "CGame.h"

const char *CObjectPrototype::MaterialNames[TOTAL_MATERIALS] =
{
	"*Reserved*",
	"Wood",
	"Steel",
	"Spanish Steel",
	"Amantite",
	"Leather",
	"Gold",
	"Nickle"
};

const char *CObjectPrototype::TypeNames[] = 
{
	"NONE",
	"FOOD",
	"LIGHT",
	"SCROLL",
	"WAND",
	"STAFF",
	"ARMOR",
	"DRINKCONTAINER",
	"CONTAINER",
	"FOUNTAIN",
	"WEAPON",
	"POTION",
	"PORTAL",
	"reserved",
	"TRANSPORT"
};

const char *CObjectPrototype::AffectNames[] = 
{
	"NONE",
	"STRADD",
	"STR",
	"AGI",
	"DEX",
	"INT",
	"WIS",
	"CON",
	"CHA",
	"POW",
	"LUCK",
	"CLASS",
	"LEVEL",
	"AGE",
	"WIEGHT",
	"HEIGHT",
	"MANA",
	"HITPOINTS",
	"MOVE",
	"EXP",
	"AC",
	"HITROLL",
	"DAMROLL",
	"SAVE_PARA",
	"SAVE_ROD",
	"SAVE_BREATHE",
	"SAVE_SPELL"
};


CObjectPrototype::CObjectPrototype(CGame *pGame)
:m_pGame(pGame)
{
	ObjAffects.SetDeleteData();
	ObjExtraDesc.SetDeleteData();
	Val0 = -1;
	Val1 = -1;
	Val2 = -1;
	Val3 = -1;
	Val4 = -1;
}


CObjectPrototype::CObjectPrototype(CObjectPrototype *pObj)
:m_pGame(pObj->GetGame())
{
	ObjAffects.SetDeleteData();
	ObjExtraDesc.SetDeleteData();
	AffBit = pObj->AffBit;
	Cost = pObj->Cost;
	Material = pObj->Material;
	NumberInGame = 0;
	ObjAlias = pObj->ObjAlias;
	ObjDesc = pObj->ObjDesc;
	ObjName = pObj->ObjName;
	ObjSittingDesc = pObj->ObjSittingDesc;
	ObjType = pObj->ObjType;
	Val0 = pObj->Val0;
	Val1 = pObj->Val1;
	Val2 = pObj->Val2;
	Val3 = pObj->Val3;
	Val4 = pObj->Val4;
	Vnum = pObj->Vnum;
	WearBit = pObj->WearBit;
	Weight = pObj->Weight;
	POSITION pos = pObj->ObjAffects.GetStartPosition();
	while(pos)
	{
		ObjAffects.Add(new sObjectWearAffects(pObj->ObjAffects.GetNext(pos)));
	}
	pos = pObj->ObjExtraDesc.GetStartPosition();
	while(pos)
	{
		ObjExtraDesc.Add(new CObjectDesc(pObj->ObjExtraDesc.GetNext(pos)));
	}
}


CObjectPrototype::CObjectPrototype(CGame *pGame,short nAffect,CRoom *pToRoom, int nPulses)
:m_pGame(pGame)
{
	ObjAffects.SetDeleteData();
	ObjExtraDesc.SetDeleteData();
	AffBit = CObject::ITEM_NOT_TAKEABLE;
	Cost = 0;
	Material = 0;
	NumberInGame = 0;
	if(nAffect!=CObject::OBJ_AFFECT_DELAYED_BLAST_FIREBALL)
	{
		Val0 = pToRoom->GetVnum();
		Val1 = nPulses;
		if(nAffect == CPortal::WORMHOLE)
		{
			ObjAlias = "portal wormhole worm hole";
			ObjName = "&Lwormhole&n";
			ObjSittingDesc = "&LA Wormhole is here tearing itself apart&n";
			ObjType = ITEM_TYPE_PORTAL;
			Vnum = CPortal::WORMHOLE;
		}
		else if(nAffect == CPortal::MOONWELL)
		{
			ObjAlias = "portal moon well moonwell";
			ObjName = "&WMoonwell&n";
			ObjSittingDesc = "&GA &WMoonwell &Gstirs here, bubbling and churning.&n";
			ObjType = ITEM_TYPE_PORTAL;
			Vnum = CPortal::MOONWELL;
		}
		else if(nAffect == CPortal::GUKS_MAGIC_PORTAL)
		{
			ObjAlias = "portal guks magic";
			ObjName = "&Rmagic &Yportal&n";
			ObjSittingDesc = "&YA Po&Grtal &Bof s&Rwirling &Gcolors &Wis &Mhere.&n";
			ObjType = ITEM_TYPE_PORTAL;
			Vnum = CPortal::GUKS_MAGIC_PORTAL;
		}
	}
	else// if(nAffect == CObject::OBJ_AFFECT_DELAYED_BLAST_FIREBALL)
	{
		ObjAlias = "fireball fire ball";
		ObjName = "&Rfireball&n";
		ObjSittingDesc = "&RA h&Wo&Rt fireball sits here &rburning&R.";
		ObjType = ITEM_TYPE_SPELL_OBJ;
		Vnum = -2;
		Val0 = 0;
		Val1 = 0;
	}
	Val2 = 0;
	Val3 = 0;
	Val4 = 0;
	WearBit = 0;
	Weight = 0;
}


CObjectPrototype::CObjectPrototype(CGame *pGame,short nType, int nDam)
:m_pGame(pGame)
{
	ObjAffects.SetDeleteData();
	ObjExtraDesc.SetDeleteData();
	AffBit = CObject::ITEM_NOT_TAKEABLE;
	Cost = 0;
	Material = 0;
	NumberInGame = 0;
	Vnum = -2;
	ObjAlias = "fireball fire ball";
	ObjName = "&Rfireball&n";
	ObjSittingDesc = "&RA &whot&R fireball sits here &rb&Ru&wr&Wn&wi&Rn&rg&R.";
	ObjType = ITEM_TYPE_SPELL_OBJ;
	Val1 = nDam;
	Val2 = 0;
	Val3 = 0;
	Val4 = 0;
	WearBit = 0;
	Weight = 0;
}


CObjectPrototype::~CObjectPrototype()
{
	for (objresources_t::iterator it = m_recipe.begin(); it != m_recipe.end(); it++)
	{
		db::Obj_resource *tmp = static_cast<db::Obj_resource *>(*it);
		delete tmp;
	}
}


const char *CObjectPrototype::GetTypeName(short n)
{
	return n>=TOTAL_OBJECT_TYPES ? "illegal type" : TypeNames[n];
}
const char *CObjectPrototype::GetMaterialName(short n)
{
	return n>=TOTAL_MATERIALS ? "illegal material" : MaterialNames[n];
}
const char *CObjectPrototype::GetAffectName(short n)
{
	return n>=TOTAL_AFFECT_TYPES ? "illegal affect type" : AffectNames[n];
}


void CObjectPrototype::ReadObj(Database *db,db::Obj &obj)
{
	Query q(db);
	CAscii tmp;
	char sql[200];

	Vnum = obj.num;
	ObjAlias = obj.alias;
	ObjName = obj.name;
	ObjDesc = obj.description;
	ObjSittingDesc = obj.sitting_description;
	if (GetGame() -> NoColor())
	{
		ObjName.Colorize(false);
		ObjDesc.Colorize(false);
		ObjSittingDesc.Colorize(false);
	}
	ObjType = obj.object_type;

	AffBit = tmp.AsciiConvert( obj.affbit );
	WearBit = tmp.AsciiConvert( obj.wearbit );

	Val0 = obj.val0;
	Val1 = obj.val1;
	Val2 = obj.val2;
	Val3 = obj.val3;
	Val4 = obj.val4;
	switch (ObjType)
	{
	case ITEM_TYPE_CONTAINER:
	case ITEM_TYPE_FOUNTAIN:
		Val1 = tmp.AsciiConvert(obj.strval1);
		break;
	}
	Weight = obj.weight;
	Cost = obj.cost;
	Material = obj.material;

	sprintf(sql,"select * from obj_extra where obj=%d", Vnum);
	q.get_result(sql);
	while (q.fetch_row())
	{
		db::Obj_extra x(db,&q);
		CObjectDesc *ExtraDesc = new CObjectDesc;
		ExtraDesc->KeyWord = x.keyword;
		ExtraDesc->ExtraDesc = x.description;
		ObjExtraDesc.Add( ExtraDesc );
	}
	q.free_result();

	sprintf(sql,"select * from obj_affect where obj=%d", Vnum);
	q.get_result(sql);
	while (q.fetch_row())
	{
		db::Obj_affect x(db,&q);
		sObjectWearAffects *NewAffect = new sObjectWearAffects();
		NewAffect->Location = x.location;
		NewAffect->Affect = x.affect;
		ObjAffects.Add( NewAffect );
	}
	q.free_result();

	sprintf(sql,"select * from obj_resource where obj=%d", Vnum);
	q.get_result(sql);
	while (q.fetch_row())
	{
		m_recipe.push_back(new db::Obj_resource(db,&q));
	}
	q.free_result();
}


/////////////////////////
//	ReadObj
//	Reads in an object from an 
//	ascii file.
////////////////////////
void CObjectPrototype::ReadObj(CGame *pGame,Database *db,CAscii &Infile)
{
	MyString temp(MAX_FILE_STRING_SIZE);
	char flag1[33] = {'\0'};
	char flag2[33] = {'\0'};
//	int counter=0;
#ifdef DODB
	Query q(db);
	char sql[400];
	char slask[400];
#endif // DODB
/*
 num
 alias (text)
 name (text)
 description (text)
 sitting description (text)
 object type | AffBit | WearBit
 Val0
 Val1
 Val2
 Val3
 Val4
 Weight
 Cost
 Material
 <Extra keyword, extra description
 <Affect location, affect
*/
	// object# or ENDFILE
	Infile.getline(temp,MAX_FILE_STRING_SIZE);
	if(temp==(MyString)"ENDFILE")
	{
		if(Infile.rdstate()!=ios::eofbit)
		{
			Infile.clear(ios::eofbit);
		}
		CError Err(NONERROR,"End File Marker");
		throw &Err;
	}
	Vnum = atol((char *)temp);
/*
	istringstream one((char *)temp); //,MAX_FILE_STRING_SIZE);
	one >> Vnum;
	if(!one.good())
	{
		CError Err(NON_CRITICAL_ERROR,"Object has no vnum.\r\n");
		throw &Err;
	}
*/
	// object alias
	Infile.ReadFileString(ObjAlias,'~');
	if(ObjAlias.IsEmpty())
	{
		ErrorLog << "Object #: " << Vnum << " has no Alias." << endl;
	}
	else 
	if(ObjAlias.GetLength() > MAX_OBJ_SHORT_STR)
	{
		ErrorLog << "Object #: " << Vnum << " alias is too long name will be truncated if saved." << endl;
	}
	// object name
	Infile.ReadFileString(ObjName,'~');
	if(ObjName.GetLength()>MAX_OBJ_SHORT_STR)
	{
		ErrorLog << "Object #: " << Vnum << " Object Name is too long it will be truncated when saved." << endl;
	}
	// object description
	Infile.ReadFileString(ObjDesc,'~');
	if(ObjDesc.GetLength()>MAX_OBJ_LONG_STR)
	{
		ErrorLog << "Object #: " << Vnum << "Object description too long it will be truncated when saved." << endl;
	}
	// object sitting description
	Infile.ReadFileString(ObjSittingDesc,'~');
	if(ObjSittingDesc.GetLength() > MAX_OBJ_LONG_STR)
	{
		ErrorLog << "Object #: " << Vnum << "object sitting description too long it will be truncated when saved." << endl;
	}
	// object type, flag1 - AffBit, flag2 - WearBit
	Infile.Numeric("%b %s %s",&ObjType,flag1,flag2);
	MyString strTmpAffBit = flag1;
	MyString strTmpWearBit = flag2;
	MyString strTmp = "";
	AffBit = Infile.AsciiConvert(flag1);
	WearBit = Infile.AsciiConvert(flag2);
	// beroende p. object type:
	switch(ObjType)
	{
	case ITEM_TYPE_ARMOR:
		Infile.Numeric("%d",&Val0);
		break;
	case ITEM_TYPE_PORTAL: //portal time loaded in minutes we need it in pulses
		Infile.Numeric("%d %d",&Val0,&Val1);
		if(Val1!=-1) //-1 is forever
		{
			Val1 = CMudTime::PULSES_PER_REAL_MIN * Val1;
		}
		break;
	case ITEM_TYPE_LIGHT: 
		Infile.Numeric("%d %d",&Val0,&Val1);
		break;
	case ITEM_TYPE_SCROLL:
	case ITEM_TYPE_STAFF:
	case ITEM_TYPE_POTION:
	case ITEM_TYPE_WEAPON: 
	case ITEM_TYPE_TRANSPORT:
		Infile.Numeric("%d %d %d %d %d",&Val0,&Val1,&Val2,&Val3,&Val4);
		break;
	case ITEM_TYPE_CONTAINER:
	case ITEM_TYPE_FOUNTAIN:
		Infile.Numeric("%d %s %d",&Val0,flag1,&Val2);
		strTmp = flag1;
		Val1 = Infile.AsciiConvert(flag1);
		break;
	default:
		ErrorLog << "Illegal item type is obj # : " << Vnum  << endl;
		break;
	}
	// object weight, object cost, object material
	Infile.Numeric("%d %d %b",&Weight,&Cost,&Material);
	// save db
#ifdef DODB
	db::Obj obj(db, Vnum);
	if (pGame -> GenerateDB())
	{
	obj.num = Vnum;
	strcpy(obj.alias, ObjAlias);
	strcpy(obj.name, ObjName);
	strcpy(obj.description, ObjDesc);
	strcpy(obj.sitting_description, ObjSittingDesc);
	obj.object_type = ObjType;
	//obj.affbit = AffBit;
	//obj.wearbit = WearBit;
	strcpy(obj.affbit, strTmpAffBit);
	strcpy(obj.wearbit, strTmpWearBit);
	strcpy(obj.strval1, strTmp);
	obj.val0 = Val0;
	obj.val1 = Val1;
	obj.val2 = Val2;
	obj.val3 = Val3;
	obj.val4 = Val4;
	obj.weight = Weight;
	obj.cost = Cost;
	obj.material = Material;
	obj.save();
	obj.num = Vnum;
	} // GenerateDB
#endif // DODB
	// ... read until ENDOBJ, recognizes ENDOBJ / EXTRA / AFFECTS
	Infile.getline(temp,MAX_FILE_STRING_SIZE);
	sObjectWearAffects *NewAffect;
	while(temp!=(MyString)"ENDOBJ" && Infile.good())
	{
		if(temp==(MyString)"EXTRA")
		{
			CObjectDesc *ExtraDesc;
			// EXTRA
			//  object extra keyword
			//  object extra description
			ObjExtraDesc.Add((ExtraDesc = new CObjectDesc));
			Infile.ReadFileString(ExtraDesc->KeyWord,'~');
			Infile.ReadFileString(ExtraDesc->ExtraDesc,'~');
			// db
#ifdef DODB
			if (pGame -> GenerateDB())
			{
			strcpy(slask,ExtraDesc -> KeyWord);
			q.safestr(slask);
			sprintf(sql,"select * from obj_extra where obj=%ld and keyword='%s'",
				obj.num,slask);
			db::Obj_extra x(db,sql);
			x.obj = obj.num;
			strcpy(x.keyword, ExtraDesc -> KeyWord);
			strcpy(x.description, ExtraDesc -> ExtraDesc);
			x.save();
			} // GenerateDB
#endif // DODB
			//
			Infile.getline(temp,MAX_FILE_STRING_SIZE);
		}
		else 
		if (temp==(MyString) "AFFECTS")
		{
			// AFFECTS
			//  object affect location, object affect
			// ...until ENDOBJ
			Infile.getline(temp,MAX_FILE_STRING_SIZE);
			while(temp!=(MyString)"ENDOBJ")
			{
				MyString strNum;
				ObjAffects.Add((NewAffect = new sObjectWearAffects()));
				strNum = temp.GetWordAfter(0);
				NewAffect->Location = (short) strNum.MakeInt();
				strNum = temp.GetWordAfter(1);
				NewAffect->Affect = (short) strNum.MakeInt();
				// db
#ifdef DODB
				if (pGame -> GenerateDB())
				{
				sprintf(sql,"select * from obj_affect where obj=%ld and location=%d and affect=%d",
					obj.num,NewAffect -> Location,NewAffect -> Affect);
				db::Obj_affect x(db,sql);
				x.obj = obj.num;
				x.location = NewAffect -> Location;
				x.affect = NewAffect -> Affect;
				x.save();
				} // GenerateDB
#endif // DODB
				//
				Infile.getline(temp,MAX_FILE_STRING_SIZE);
			}
		}
		else
		{
			//
			Infile.getline(temp,MAX_FILE_STRING_SIZE);
		}
	}
}

void CObjectPrototype::VStat(MyString & ToChar)
{
	static MyString strFormat(
		"Name: %s       Material: %s\r\n"
		"Val0: %d  Val1: %d  Val2: %d Val3: %d Val4:%d\r\n"
		"Weight: %d		Cost: %d\r\n"
		"Object Type: %s\r\n"
		"Object Alias: %s\r\n"
		"Obj Description: %s \r\n"
		"Affects on Object:\r\n");

	ToChar.Format(strFormat,(char *)ObjName,GetMaterialName(Material),
		Val0,Val1,Val2,Val3,Val4,Weight,Cost,
		GetTypeName(ObjType),(char *)ObjAlias,
		(char *)ObjDesc);

	sObjectWearAffects *pAff;
	POSITION pos = ObjAffects.GetStartPosition();
	MyString str;
	while(pos)
	{
		pAff = ObjAffects.GetNext(pos);
		str.Format("%s %d\r\n",
			GetAffectName(pAff->Location),
			pAff->Affect);
		ToChar+=str;
	}
}

