/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "stdinclude.h"
#include <stdio.h>
#include <vector>
#include <string>
#include "CTimerFactory.h"
#include "CTimer.h"


CTimerFactory::CTimerFactory()
{
}


CTimerFactory::~CTimerFactory()
{
}


CTimer *CTimerFactory::AddTimer(CTimer *t)
{
	m_timers.insert(m_timers.end(),t);
	return t;
}


void CTimerFactory::RemoveTimer(CTimer *t)
{
	for (timers_t::iterator it = m_timers.begin(); it != m_timers.end(); it++)
	{
		CTimer *tmp = static_cast<CTimer *>(*it);
		if (tmp == t)
		{
			delete tmp;
			m_timers.erase(it);
			break;
		}
	}
}


void CTimerFactory::Pulse(long /*lPulse*/)
{
	bool repeat = false;
	for (timers_t::iterator it = m_timers.begin(); it != m_timers.end(); it++)
	{
		CTimer *tmp = static_cast<CTimer *>(*it);

		tmp -> Tick();
		if (tmp -> KillThis())
		{
			repeat = true;
		}
	}
	while (repeat)
	{
		repeat = false;
		for (timers_t::iterator it = m_timers.begin(); it != m_timers.end(); it++)
		{
			CTimer *tmp = static_cast<CTimer *>(*it);
			if (tmp -> KillThis())
			{
				delete tmp;
				m_timers.erase(it);
				repeat = true;
				break;
			}
		}
	}
}


void CTimerFactory::DeleteTimers()
{
	for (timers_t::iterator it = m_timers.begin(); it != m_timers.end(); it++)
	{
		CTimer *tmp = static_cast<CTimer *>(*it);
		delete tmp;
	}
}


timers_t &CTimerFactory::GetList()
{
	return m_timers;
}



