/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <mysql/mysql.h>
#include <libsql++.h>
#include <string>
using std::string;
#include <libcgi++.h>
#include <libwolfshade.h>
#include <libgd.h>

#include "BaseForm.h"
#include "WolfshadeWeb.h"
#include "Map.h"
#include "MapForm.h"


namespace cgi
{


MapForm::MapForm(Web *web)
:BaseForm(web, "mapform")
,m_vzone(0)
{
	Query q(GetDatabase());

	q.get_result("select * from room where virtual_room<>0");
	while (q.fetch_row())
	{
		db::Room *tmp = new db::Room(GetDatabase(),&q);
		m_vroom[tmp -> num] = tmp;
	}
	q.free_result();

}


MapForm::~MapForm()
{
	for (rooms_t::iterator it = m_vroom.begin(); it != m_vroom.end(); it++)
	{
		delete (*it).second;
	}
}


void MapForm::Process()
{
	WolfshadeWeb *web = static_cast<WolfshadeWeb *>(GetWeb());
	char action[200];
	
	if (GetForm() -> getvalue("mapform",action,200))
	{
		char slask[100];
		if (!strcmp(action,"vzone"))
		{
			GetForm() -> getvalue("vzone",slask,100);
			m_vzone = atol(slask);
		}
		if (!strcmp(action,"map"))
		{
			char sql[200];
			// 'map' is vzone num - got ya
			GetForm() -> getvalue("map",slask,100);
			long map = atol(slask);
			int level = 0;
			sprintf(sql,"select * from vmap where vzone=%ld and level=%d",map,level);
			db::Vmap x(GetDatabase(),sql);
			if (!x.pic)
			{
				CreatePicture("/tmp/map_pic.gif",map,level);
				x.pic = AddPic(x.pic,"/tmp/map_pic.gif");
				x.save();
			}
			db::Pic pic(GetDatabase(),x.pic);
			web -> SetPic(pic.pic,pic.ext);
		}
	}
}


void MapForm::Select(const string &)
{
}


void MapForm::List()
{
}


void MapForm::Display(long)
{
	WolfshadeWeb *web = static_cast<WolfshadeWeb *>(GetWeb());
	Database *db = GetDatabase();
	Query q(db);

	if (m_vzone)
	{
		printf("<img src=%s?%s=map&map=%ld&lvl=0>",
			GetCgiName().c_str(),
			GetFormName().c_str(),m_vzone);
	}
}


void MapForm::CreatePicture(const string &filename,long vzonenum,int level)
{
	db::Vzone vz(GetDatabase(),vzonenum);
	Map map(GetDatabase(),vzonenum,level);
	gd::Image img(vz.sizex,vz.sizey,vz.sizex,vz.sizey);
	int black = img.AddColor(0,0,0);
	int colors[256];

	for (int i = 0; i < 256; i++)
		colors[i] = 0;
	colors[0] = black;
	for (int i = 'a'; i < 'z'; i++)
	{
		int tmp = i - 'a';
		int intensity = 4 * tmp / 8;
		int r = (tmp & 1) ? 0x33 : 0;
		int g = (tmp & 2) ? 0x33 : 0;
		int b = (tmp & 4) ? 0x33 : 0;
		r *= intensity;
		g *= intensity;
		b *= intensity;
		colors[i] = img.AddColor(r,g,b);
	}
	for (int y = 0; y < vz.sizey; y++)
	{
		for (int x = 0; x < vz.sizex; x++)
		{
			char c = map.GetAt(x,y);
			if (!colors[c])
				colors[c] = img.AddColor(random()%256,random()%256,random()%256);
			img.Line(x,y,x,y,colors[c]);
		}
	}
	{
		FILE *fil = fopen(filename.c_str(),"wb");
		if (fil)
		{
			img.SaveTo(fil);
			fclose(fil);
		}
	}
}


} // namespace cgi


