/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <mysql/mysql.h>
#include <libsql++.h>
#include <string>
using std::string;
#include <libcgi++.h>
#include <libwolfshade.h>

#include "BaseForm.h"
#include "WolfshadeWeb.h"
//#include "CMemoryAllocator.h"
//#include "MyString.h"
#include "MobForm.h"


namespace cgi
{


MobForm::MobForm(Web *web)
:BaseForm(web, "mobform")
,m_edit(0)
,m_error(false)
{
	Query q(GetDatabase());

	q.get_result("select * from classes");
	while (q.fetch_row())
	{
		db::Classes *tmp = new db::Classes(GetDatabase(),&q);
		m_class[tmp -> num] = tmp;
	}
	q.free_result();

	q.get_result("select * from race");
	while (q.fetch_row())
	{
		db::Race *tmp = new db::Race(GetDatabase(),&q);
		m_race[tmp -> num] = tmp;
	}
	q.free_result();
}


MobForm::~MobForm()
{
	for (classes_t::iterator it = m_class.begin(); it != m_class.end(); it++)
	{
		delete (*it).second;
	}
	for (races_t::iterator it = m_race.begin(); it != m_race.end(); it++)
	{
		delete (*it).second;
	}
}


void MobForm::Process()
{
	char action[200];
	
	if (GetForm() -> getvalue("mobform",action,200))
	{
		if (!strcmp(action,"edit"))
		{
			char slask[100];
			GetForm() -> getvalue("edit",slask,100);
			m_edit = atol(slask);
		}
		else
		if (!strcmp(action," Save "))
		{
			long mobflags = 0;
			long loadpos = 0;
			long defpos = 0;
			char slask[100];
			char value[256];
			GetForm() -> getvalue("num",slask,100);
			long num = atol(slask);
			GetForm() -> getvalue("onum",slask,100);
			long onum = atol(slask);
			db::Mob mob(GetDatabase(),num);

			if (num != onum && mob.num)
			{
				m_error = true;
			}
			else
			{
// alias, name, dpd, descr, longdescr
				GetForm() -> EnableRaw( true );
				mob.num = num;
				GetForm() -> getvalue("alias",mob.alias,255);
				GetForm() -> getvalue("name",mob.name,255);
				GetForm() -> getvalue("dpd",mob.default_pos_description,255);
				GetForm() -> getvalue("descr",mob.description,255);
				GetForm() -> getvalue("longdescr",mob.long_description,255);
				// loop to get multiple checkbox values
				GetForm() -> getfirst(slask,100,value,256);
				while (*slask)
				{
					if (!strcmp(slask,"mobflags"))
						mobflags |= atol(value);
					if (!strcmp(slask,"loadpos"))
						loadpos |= atol(value);
					if (!strcmp(slask,"defpos"))
						defpos |= atol(value);
					//
					GetForm() -> getnext(slask,100,value,256);
				}
				sprintf(mob.mobflags,"%ld",mobflags);
				GetForm() -> getvalue("prof",mob.proficiencysphere,255);
				GetForm() -> getvalue("alignment",slask,100);
				mob.alignment = atol(slask);
				GetForm() -> getvalue("race",slask,100);
				mob.race = atol(slask);
				GetForm() -> getvalue("classtype",slask,100);
				mob.classtype = atol(slask);
				GetForm() -> getvalue("level",slask,100);
				mob.level = atol(slask);
				//
				GetForm() -> getvalue("thaco",slask,100);
				mob.thaco = atol(slask);
				GetForm() -> getvalue("ac",slask,100);
				mob.ac = atol(slask);
				GetForm() -> getvalue("minhits",slask,100);
				mob.minhits = atol(slask);
				GetForm() -> getvalue("numofdice",slask,100);
				mob.numofdice = atol(slask);
				GetForm() -> getvalue("dicetype",slask,100);
				mob.dicetype = atol(slask);
				GetForm() -> getvalue("copper",slask,100);
				mob.copper = atol(slask);
				GetForm() -> getvalue("exp",slask,100);
				mob.experience = atol(slask);
				sprintf(mob.loadpos, "%ld", loadpos);
				sprintf(mob.defaultpos, "%ld", defpos);
				GetForm() -> getvalue("sex",slask,100);
				mob.sex = atol(slask);
				//
				mob.save();
				m_edit = num;
			}
		}
	}
}


void MobForm::Select(const string &)
{
}


void MobForm::List()
{
}


void MobForm::Display(long)
{
	WolfshadeWeb *web = static_cast<WolfshadeWeb *>(GetWeb());
	Database *db = GetDatabase();
	Query q(db);
	long zonesize = 0;
	char sql[200];

	zonesize = EditingZone();

	if (!m_edit)
		web -> DisplayStartAntalForm();

	if (m_error)
	{
		printf("<h3>New Mob# is already taken</h3>");
	}

	{
		if (m_edit)
			sprintf(sql,"select * from mob where num=%ld",m_edit);
		else
		{
			if (GetRace() || GetClass())
			{
				char slask[100];

				sprintf(sql,"select * from mob");
				strcpy(slask," where");
				if (GetRace())
				{
					sprintf(sql + strlen(sql),"%s race=%ld",slask,GetRace());
					strcpy(slask," and");
				}
				if (GetClass())
				{
					sprintf(sql + strlen(sql),"%s classtype=%ld",slask,GetClass());
					strcpy(slask," and");
				}
				if (GetZone())
				{
					sprintf(sql + strlen(sql),"%s num>=%ld and num<%ld",
						slask,
						GetZone() * 100,GetZone() * 100 + zonesize);
					strcpy(slask," and");
				}
				strcat(sql," order by num");
			}
			else
			if (GetZone())
			{
				sprintf(sql,"select * from mob where num>=%ld and num<%ld order by num",
					GetZone() * 100,GetZone() * 100 + zonesize);
			}
			else
			{
				sprintf(sql,"select * from mob where num>=%ld order by num limit %ld",GetStart(),GetAntal());
			}
		}
		q.get_result(sql);
		char header[100];
		
		if (m_edit)
			sprintf(header,"&nbsp;Editing Mob#%ld",m_edit);
		else
			sprintf(header,"&nbsp;Mobs (%ld mobs displayed)",q.num_rows());
		Table tbl("#000000", 3, header);
		printf("<tr bgcolor=#ffffff>");
		printf("<th align=left>&nbsp;Mob | Vnum</th>");
		printf("<th align=left>&nbsp;Race</th>");
		printf("<th align=left>&nbsp;Class</th>");
		printf("</tr>");

		while (q.fetch_row())
		{
			db::Mob mob(db,&q);

			printf("<tr bgcolor=#808080>");
			printf("<td bgcolor=#808080>");
			ShowMob(mob,true);
			printf("</td>");
			printf("<td>&nbsp;%s&nbsp;</td>",htmlcolor(m_race[mob.race] -> name));
			printf("<td>&nbsp;%s&nbsp;</td>",htmlcolor(m_class[mob.classtype] -> name));
			printf("</tr>");
		}
		q.free_result();
	}
	printf("<br>");

	if (m_edit)
	{
		db::Mob mob(db,m_edit);

		// references to this mob
		{
/*
CREATE TABLE zone_m (
  num int(11) NOT NULL auto_increment,
  zone int(11) NOT NULL default '0',
  mobvnum int(11) NOT NULL default '0',
  max int(11) NOT NULL default '0',
  roomvnum int(11) NOT NULL default '0',
  PRIMARY KEY  (num),
  KEY zone (zone,roomvnum,mobvnum)
) TYPE=MyISAM;
*/
			sprintf(sql,"select * from zone_m where mobvnum=%ld",mob.num);
			q.get_result(sql);
			if (q.num_rows())
			{
				Table tbl("#000000", 3, "&nbsp;Zone references to this Mob");
				printf("<tr bgcolor=#ffffff>");
				printf("<th>Zone</th>");
				printf("<th>Room</th>");
				printf("<th>Max</th>");
				printf("</tr>");
				while (q.fetch_row())
				{
					db::Zone_m x(db,&q);
					db::Zone zone(db,x.zone);
					db::Room room(db,x.roomvnum);
					printf("<tr bgcolor=#808080>");
//					printf("<td>&nbsp;#%ld: %s&nbsp;</td>",zone.num,htmlcolor(zone.name));
					ShowZone(zone,true,true);
//					printf("<td>&nbsp;#%ld: %s&nbsp;</td>",room.num,htmlcolor(room.name));
					ShowRoom(room,true,true);
					printf("<td align=right>&nbsp;%ld&nbsp;</td>",x.max);
					printf("</tr>");
				}
			}
			if (q.num_rows())
			{
				printf("<br>");
			}
			q.free_result();
/*
CREATE TABLE shop (
  num int(11) NOT NULL default '0',
  percentsell decimal(7,2) NOT NULL default '0.00',
  percentbuy decimal(7,2) NOT NULL default '0.00',
  shopkeeper int(11) NOT NULL default '0',
  loot int(11) NOT NULL default '0',
  PRIMARY KEY  (num)
) TYPE=MyISAM;
*/
			sprintf(sql,"select * from shop where shopkeeper=%ld",mob.num);
			q.get_result(sql);
			if (q.num_rows())
			{
				Table tbl("#000000", 3, "&nbsp;Shopkeeper");
				printf("<tr bgcolor=#ffffff>");
				printf("<th>Zone</th>");
				printf("<th>Room</th>");
				printf("</tr>");
				while (q.fetch_row())
				{
					db::Shop shop(db,&q);
					db::Room room(db,shop.num);
					db::Zone zone(db,room.zone);
					printf("<tr bgcolor=#808080>");
//					printf("<td>&nbsp;#%ld: %s&nbsp;</td>",zone.num,htmlcolor(zone.name));
					ShowZone(zone,true,true);
//					printf("<td>&nbsp;#%ld: %s&nbsp;</td>",room.num,htmlcolor(room.name));
					ShowRoom(room,true,true);
					printf("</tr>");
				}
			}
			if (q.num_rows())
			{
				printf("<br>");
			}
			q.free_result();
		}

		// edit
		{
			Table tbl("#000000", 3);
			long mobflags = asciiconvert(mob.mobflags);
			long loadpos = asciiconvert(mob.loadpos);
			long defpos = asciiconvert(mob.defaultpos);

			printf("<form action=%s method=post>",GetCgiName().c_str());
			printf("<input type=hidden name=onum value=%ld>",mob.num);

			inputnum("Vnum", "num", mob.num);
			inputstr("Alias","alias",40,mob.alias);
			inputstr("MobName","name",60,mob.name,true);
			textarea("Default Pos Description","dpd",mob.default_pos_description,true);
			textarea("Description","descr",mob.description,true);
			textarea("Long Description","longdescr",mob.long_description,true);
			inputbit(q,"MobFlags","mobflags","select num,name from mob_action order by name",mobflags,4);
			// class specific:
			printf("<tr bgcolor=#ffffc0>");
			printf("<td>Proficiencies</td>");
			printf("<td><input type=text size=20 name=prof value=\"%s\">",mob.proficiencysphere);
			printf(" (Class specific)");
			printf("</td>");
			printf("</tr>");

			// Alignment
			inputnum("Alignment", "alignment", mob.alignment);
			inputselect(q,"Race","race","select num,alias from race where num>0 order by alias",mob.race);
			inputselect(q,"ClassType","classtype","select num,alias from classes where num>0 order by alias",mob.classtype);
			inputnum("Level", "level", mob.level);
			inputnum("Thaco", "thaco", mob.thaco);
			inputnum("AC", "ac", mob.ac);
			inputnum("MinHits", "minhits", mob.minhits);
			inputnum("NumOfDice", "numofdice", mob.numofdice);
			inputnum("DiceType", "dicetype", mob.dicetype);
			inputnum("Copper", "copper", mob.copper);
			inputnum("Experience", "exp", mob.experience);
			inputbit(q,"LoadPos","loadpos","select num,name from mob_pos order by name",loadpos,4);
			inputbit(q,"DefaultPos","defpos","select num,name from mob_pos order by name",defpos,4);
			inputselect(q,"Sex","sex","select num,name from mob_sex order by num",mob.sex);
			inputsubmit(" Save ");

			printf("</form>");
		}
		printf("<br>");
	}
}


} // namespace cgi



/*
CREATE TABLE mob (
  num int(11) NOT NULL default '0',
  alias varchar(255) NOT NULL default '',
  name varchar(255) NOT NULL default '',
  default_pos_description varchar(255) NOT NULL default '',
  description varchar(255) NOT NULL default '',
  long_description varchar(255) NOT NULL default '',
  MobFlags varchar(255) NOT NULL default '0',
  ProficiencySphere varchar(255) NOT NULL default '0',
  Alignment int(11) NOT NULL default '0',
  Race int(11) NOT NULL default '0',
  ClassType int(11) NOT NULL default '0',
  Level int(11) NOT NULL default '0',
  Thaco int(11) NOT NULL default '0',
  AC int(11) NOT NULL default '0',
  MinHits int(11) NOT NULL default '0',
  NumOfDice int(11) NOT NULL default '0',
  DiceType int(11) NOT NULL default '0',
  PlusDam int(11) NOT NULL default '0',
  Copper int(11) NOT NULL default '0',
  Experience int(11) NOT NULL default '0',
  LoadPos varchar(255) NOT NULL default '0',
  DefaultPos varchar(255) NOT NULL default '0',
  Sex int(11) NOT NULL default '0',
  PRIMARY KEY  (num)
) TYPE=MyISAM;
*/

