//////////////////////////////////////////////////////////////
//Wolfshade MUD server
//Copyright (C) 1999 Demetrius and John Comes
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
/*
*  Header File for MySocket
*	Socket handling
*		Handals all socket manipulates moves commands from sockets
*		to descriptor command buffer.
*		Manages charIntermediate
*/


#ifndef _SOCKET_H
#define _SOCKET_H

#include <map>
#ifndef _WOLFSHADE_WINDOWS_
#include <netinet/in.h> //INADDR.ANY header
#endif
#include "CMutex.h"

class CListenThread;
typedef CLinkList<sPlayerDescriptor> llpdesc_t;
typedef CLinkList<CCharIntermediate> llcchari_t;
typedef std::map<SOCKET,sPlayerDescriptor *> sockmap_t;
class MySocketsThread;

class MySocket
{

public:
  			~MySocket();
  			MySocket(CGame *,MyString strOpts);

  CGame *		GetGame() { return m_pGame; }

  int 			NonBlock(SOCKET s);
  void 			BootFileStrings();
  void 			SignalTrap();
  void 			AddCharFromDeath(CCharacter *pChar);
  void 			KillSocket(sPlayerDescriptor *pDesc);
  void 			DoNotCharsYet();
  void 			GetCommandsFromSockets();
#ifndef LISTENTHREAD
  bool 			NewConnections();
  void 			NewDescriptor();
  void 			KickOutExceptionSet();
#endif // LISTENTHREAD
  llpdesc_t &		GetDescriptors() { return Descriptors; }
  llcchari_t &		GetNotCharsYet() { return NotCharsYet; }
  llcchari_t &		GetCharQ() {return CharQ;}
  sockmap_t &		GetSockets() { return m_Sockets; }
  fd_set		GetSet() { return m_rfds; }
  sPlayerDescriptor *	AddSocket(SOCKET);
  void			RemoveSocket(SOCKET);

protected:
  static UINT 		MAX_PLAYERS;
  static UINT 		RESERVED_CONNECTIONS;

private:
  static const SOCKET 	DEFAULT_PORT;
#ifdef _WOLFSHADE_WINDOWS_
  WSADATA 		m_WsaData;
#endif
#ifdef LISTENTHREAD
  CListenThread *	m_pListenThread;
  CMutex		m_Mutex;
#else
  SOCKET 		ListenSocket;
#endif // LISTENTHREAD
  int			m_nPort;
  fd_set 		input_set, output_set, exception_set;
  struct sockaddr_in 	sa;
  short 		m_MaxPlayers;
  //variables
  struct linger 	ld;
  bool 			bNoNameServer;
  CGame *		m_pGame;
  // protected by m_Mutex
  llpdesc_t 		Descriptors;
  // protected by m_Mutex
  llcchari_t 		NotCharsYet;
  llcchari_t 		CharQ;
  sockmap_t		m_Sockets;
  fd_set		m_rfds;
  SOCKET		m_n;
#ifdef SOCKETSTHREAD
  MySocketsThread *	m_pThread;
  int			m_nSocketCount;
  vector<MySocketsThread *> m_threads;
#endif
};
#endif

