/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "stdinclude.h"
#ifdef PRAGMAHDRSTOP
#pragma hdrstop
#endif
// Database includes
#include <mysql/mysql.h>
#include <libsql++.h>
#include <libwolfshade.h>
//
#ifdef _WOLFSHADE_WINDOWS_
typedef int socklen_t;
#else
#include <sys/socket.h>  //socket functions
#include <sys/resource.h> //timeval stuct declaration
#include <errno.h> //for the type error_t
#include <fcntl.h>  //function header to nonblock a socket
#include <unistd.h> //int close()
#define closesocket close
#include <netdb.h> //hostent declaration
#include <signal.h>
#include <arpa/inet.h>
#define INVALID_SOCKET -1
#endif

#include "MySocket.h"
#include <string.h>
#include <signal.h>
#include "CWolfshadeLog.h"
#include "main.h"
#include "CCharIntermediate.h"
#include "CGame.h"
#include "CMutex.h"

#include "MySocketsThread.h"

//#define DEB(x) x; fflush(stdout);


MySocketsThread::MySocketsThread()
:CThread(false)
,m_n(0)
,m_in(0)
,m_out(0)
{
	FD_ZERO(&m_rfds);
	FD_ZERO(&m_wfds);
}


MySocketsThread::~MySocketsThread()
{
}


void MySocketsThread::Run()
{
	fd_set rfds;
	fd_set wfds;
	int n;
	struct timeval tv;

	while (!Quit())
	{
		m_lock.Lock();
		rfds = m_rfds;
		wfds = m_wfds;
		n = m_n;
		m_lock.Unlock();
		tv.tv_sec = 0;
		tv.tv_usec = 500000L;
		n = select(n + 1,&rfds,&wfds,NULL,&tv);
		if (n < 0)
		{
			// err
		}
		else
		if (!n)
		{
			// timeout
		}
		else
		{
			m_lock.Lock();
			for (sockmap_t::iterator it = m_sock.begin(); it != m_sock.end(); it++)
			{
				SOCKET s = (*it).first;
				sPlayerDescriptor *desc = (*it).second;
				if (desc)
				{
					bool locked = false;
					bool out = false;
					if (FD_ISSET(s,&rfds))
					{
						desc -> m_lock.Lock();
						locked = true;
						//
						char buf[512];
						n = recv(s, buf, 512, 0);
						if (n < 0)
						{
#ifdef _WOLFSHADE_WINDOWS_
							if (WSAGetLastError() != WSAEWOULDBLOCK) 
#else
#ifdef EWOULDBLOCK
				      			if (errno == EWOULDBLOCK)
 								errno = EAGAIN;
#endif /* EWOULDBLOCK */
							if (errno != EAGAIN)
#endif //woflshade_windows
							{
								ErrorLog << "Socket " << s << " bad read" << endl;
								desc->ChangeState(STATE_LINK_DEAD);
								m_sock[s] = NULL;
								FD_CLR(s,&m_rfds);
								FD_CLR(s,&m_wfds);
								m_out++;
								out = true;
							}
						}
						else
						if (!n)
						{
							ErrorLog << "End of file read from socket...connect closed by peer. Socket #: " << s << endl;
							desc->ChangeState(STATE_LINK_DEAD);
							m_sock[s] = NULL;
							FD_CLR(s,&m_rfds);
							FD_CLR(s,&m_wfds);
							m_out++;
							out = true;
						}
						else
						{
//							NewCommands = Com.Left(nBytesRead);
//							NewCommands.RemoveFunkyChars(); //not correct!
							//can't do this b/c clients that send 
							//each keystoke will lose all spaces!
							//NewCommands.SkipSpaces();
							buf[n] = 0;
							if((desc->CommandBuf.GetLength() + n) > MAX_COMMAND_BUFFER-1)
							{
								ErrorLog << "Socket " << s << " had too much data!" << endl;
//								desc->CommandBuf += NewCommands.Left(MAX_COMMAND_BUFFER-desc->CommandBuf.GetLength());
							}
							else
							{
								desc->CommandBuf += buf;
							}
						}
					}
					if (FD_ISSET(s,&wfds))
					{
						if (!locked)
						{
							desc -> m_lock.Lock();
							locked = true;
						}
						size_t size_to_send = desc -> m_buf.size() - desc -> m_sent;
						if (size_to_send > 0)
						{
							n = send(s, desc -> m_buf.c_str() + desc -> m_sent, size_to_send, 0);
							if (n < 0)
							{
#ifdef _WOLFSHADE_WINDOWS_
								if (WSAGetLastError() != WSAEWOULDBLOCK) 
#else
#ifdef EWOULDBLOCK
								if (errno == EWOULDBLOCK)
									errno = EAGAIN;
#endif /* EWOULDBLOCK */
								if (errno != EAGAIN)
#endif //woflshade_windows
								{
									ErrorLog << "Socket " << s << " bad write" << endl;
									desc->ChangeState(STATE_LINK_DEAD);
									m_sock[s] = NULL;
									FD_CLR(s,&m_rfds);
									FD_CLR(s,&m_wfds);
									if (!out)
									{
										m_out++;
									}
								}
							}
							else
							if (n > 0)
							{
								desc -> m_sent += n;
								if (desc -> m_sent == desc -> m_buf.size())
								{
									desc -> m_buf = "";
									desc -> m_sent = 0;
								}
							}
						}
					}
					if (locked)
					{
						desc -> m_lock.Unlock();
					}
				}
			}
			m_lock.Unlock();
		}
		if (m_in == MAX_HANDLED && m_out == m_in)
		{
			Stop();
		}
	}
	SetRunning(false);
}


void MySocketsThread::Add(SOCKET s,sPlayerDescriptor *desc)
{
	m_lock.Lock();
	m_sock[s] = desc;
	FD_SET(s, &m_rfds);
	FD_SET(s, &m_wfds);
	if (s > m_n)
	{
		m_n = s;
	}
	m_lock.Unlock();
	m_in++;
	printf("MySocketsThread::Add(%d)  m_in=%d  m_out=%d\n",s,m_in,m_out);
}


void MySocketsThread::Remove(SOCKET s)
{
	m_lock.Lock();
	for (sockmap_t::iterator it = m_sock.begin(); it != m_sock.end(); it++)
	{
		if ((*it).first == s)
		{
			m_sock[s] = NULL;
			FD_CLR(s, &m_rfds);
			FD_CLR(s, &m_wfds);
			m_out++;
			break;
		}
	}
	m_lock.Unlock();
	printf("MySocketsThread::Remove(%d)  m_in=%d  m_out=%d\n",s,m_in,m_out);
}


