/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <mysql/mysql.h>
#include <libsql++.h>
#include <string>
using std::string;
#include <libcgi++.h>
#include <libwolfshade.h>

#include "BaseForm.h"
#include "WolfshadeWeb.h"
//#include "CMemoryAllocator.h"
//#include "MyString.h"
#include "RoomForm.h"


namespace cgi
{


RoomForm::RoomForm(Web *web)
:BaseForm(web, "roomform")
,m_edit(0)
,m_error(false)
{
	Query q(GetDatabase());
	
	q.get_result("select * from zone");
	while (q.fetch_row())
	{
		db::Zone *tmp = new db::Zone(GetDatabase(),&q);
		m_zone[tmp -> num] = tmp;
	}
	q.free_result();
	
	q.get_result("select * from room_type");
	while (q.fetch_row())
	{
		db::Room_type *tmp = new db::Room_type(GetDatabase(),&q);
		m_type[tmp -> num] = tmp;
	}
	q.free_result();

	q.get_result("select * from room_size");
	while (q.fetch_row())
	{
		db::Room_size *tmp = new db::Room_size(GetDatabase(),&q);
		m_size[tmp -> num] = tmp;
	}
	q.free_result();
}


RoomForm::~RoomForm()
{
	for (zones_t::iterator it = m_zone.begin(); it != m_zone.end(); it++)
	{
		delete (*it).second;
	}
	for (types_t::iterator it = m_type.begin(); it != m_type.end(); it++)
	{
		delete (*it).second;
	}
	for (sizes_t::iterator it = m_size.begin(); it != m_size.end(); it++)
	{
		delete (*it).second;
	}
}


void RoomForm::Process()
{
	char action[200];
	
	if (GetForm() -> getvalue("roomform",action,200))
	{
		if (!strcmp(action,"edit"))
		{
			char slask[100];
			GetForm() -> getvalue("edit",slask,100);
			m_edit = atol(slask);
		}
		else
		if (!strcmp(action," Save "))
		{
			char slask[100];
			GetForm() -> getvalue("num",slask,100);
			long num = atol(slask);
			GetForm() -> getvalue("onum",slask,100);
			long onum = atol(slask);
			db::Room room(GetDatabase(),num);

			if (num != onum && room.num)
			{
				m_error = true;
			}
			else
			{
				long affects = 0;
				char value[256];
				GetForm() -> getfirst(slask,100,value,256);
				while (*slask)
				{
					if (!strcmp(slask,"affects"))
						affects |= atol(value);
					GetForm() -> getnext(slask,100,value,256);
				}
// alias, name, dpd, descr, longdescr
				GetForm() -> EnableRaw( true );
				room.num = num;
				GetForm() -> getvalue("name",room.name,255);
				int l = GetForm() -> getlength("descr");
				if (room.description)
					delete room.description;
				room.description = new char[l * 2 + 100];
				GetForm() -> getvalue("descr",room.description,255);
				GetForm() -> getvalue("zone",slask,100);
				room.zone = atol(slask);
				sprintf(room.affects,"%ld",affects);
				GetForm() -> getvalue("type",slask,100);
				room.type = atol(slask);
				GetForm() -> getvalue("size",slask,100);
				room.size = atol(slask);
				GetForm() -> getvalue("virtual_room",slask,100);
				room.virtual_room = atoi(slask);
				GetForm() -> getvalue("virtual_char",room.virtual_char,255);
				GetForm() -> getvalue("design_char",slask,100);
				sprintf(room.design_char,"%c",*slask);
				GetForm() -> getvalue("resource",slask,100);
				room.resource = atoi(slask);
				GetForm() -> getvalue("resource_speed",slask,100);
				room.resource_speed = atoi(slask);

				room.save();
				m_edit = num;
			}
		}
		else
		if (!strcmp(action," Save Extra ") ||
		    !strcmp(action," Add Extra ") )
		{
			char slask[100];
			GetForm() -> getvalue("num",slask,100);
			long num = atol(slask);
			db::Room_extra x(GetDatabase(),num);
			GetForm() -> getvalue("roomnum",slask,100);
			long roomnum = atol(slask);

			GetForm() -> EnableRaw(true);
			x.room = roomnum;
			GetForm() -> getvalue("keyword",x.keyword,255);
			GetForm() -> getvalue("descr",x.description,255);
			x.save();
			//
			m_edit = roomnum;
		}
		else
		if (!strcmp(action," Save Door ") ||
		    !strcmp(action," Add Door ") )
		{
			char slask[100];
			GetForm() -> getvalue("num",slask,100);
			long num = atol(slask);
			db::Room_door door(GetDatabase(),num);
			GetForm() -> getvalue("roomnum",slask,100);
			long roomnum = atol(slask);
			
			GetForm() -> EnableRaw(true);
			door.room = roomnum;
			GetForm() -> getvalue("dir",slask,100);
			door.direction = atol(slask);
			GetForm() -> getvalue("keyword",door.keyword,255);
			GetForm() -> getvalue("type",door.type,100);
			GetForm() -> getvalue("key",slask,100);
			door.vnumkey = atol(slask);
			GetForm() -> getvalue("toroom",slask,100);
			door.toroom = atol(slask);
			door.save();
			//
			m_edit = roomnum;
		}
		else
		if (!strcmp(action,"orientation"))
		{
// roomform=orientation, room(from), direcion, toroom
			char slask[100];
			GetForm() -> getvalue("room",slask,100);
			long roomnum = atol(slask);
			GetForm() -> getvalue("direction",slask,100);
			long dir = atol(slask);
			GetForm() -> getvalue("toroom",slask,100);
			long toroom = atol(slask);
			char sql[200];
			sprintf(sql,"select * from room_door where room=%ld and direction=%ld",
				roomnum,dir);
			db::Room_door door(GetDatabase(),sql);
			if (!toroom) // delete
			{
				door.erase();
				//
				m_edit = door.toroom;
			}
			else
			{
				if (!door.num) // new
				{
					door.vnumkey = -1;
					strcpy(door.type, "1");
				}
				door.room = roomnum;
				door.direction = dir;
				door.toroom = toroom;
				door.save();
				//
				m_edit = roomnum;
			}
		}
	}
}


void RoomForm::Select(const string &)
{
}


void RoomForm::List()
{
}


void RoomForm::Display(long)
{
	WolfshadeWeb *web = static_cast<WolfshadeWeb *>(GetWeb());
	Database *db = GetDatabase();
	Query q(db);
	Query q2(db);
	long zonesize = 0;
	char sql[200];

	zonesize = EditingZone();

	if (!m_edit)
		web -> DisplayStartAntalForm();

	if (m_error)
	{
		printf("<h3>New Room# is already taken</h3>");
	}

	// list
	{
		if (m_edit)
			sprintf(sql,"select * from room where num=%ld",m_edit);
		else
		{
			if (GetZone())
			{
				sprintf(sql,"select * from room where zone=%ld or (num>=%ld and num<%ld) order by num",
					GetZone(),GetZone() * 100,GetZone() * 100 + zonesize);
			}
			else
			{
				sprintf(sql,"select * from room where num>=%ld order by num limit %ld",GetStart(),GetAntal());
			}
		}
		q.get_result(sql);
		char header[100];

		if (m_edit)
			sprintf(header,"&nbsp;Editing Room#%ld",m_edit);
		else
			sprintf(header,"&nbsp;Rooms (%ld rooms displayed)",q.num_rows());
		Table tbl("#000000", 3, header);
		printf("<tr bgcolor=#ffffff>");
		printf("<th align=left>&nbsp;Room | Vnum</th>");
		if (!GetZone())
			printf("<th align=left>&nbsp;Zone</th>");
		printf("<th align=left>&nbsp;Type</th>");
		printf("<th align=left>&nbsp;Size</th>");
		printf("</tr>");

		while (q.fetch_row())
		{
			db::Room room(db,&q);

			printf("<tr bgcolor=#c0c0c0>");
			printf("<td bgcolor=#808080>");
			ShowRoom(room,true);
			printf("</td>");
			if (!GetZone())
			{
				if (m_zone[room.zone])
				{
					printf("<td bgcolor=#808080>");
					ShowZone(*m_zone[room.zone],true);
					printf("</td>");
				}
				else
					printf("<td bgcolor=#808080>&nbsp;%s&nbsp;</td>",m_zone[room.zone] ? htmlcolor(m_zone[room.zone] -> name) : "&lt;invalid zone&gt" );
			}
			printf("<td bgcolor=#c0c0c0>&nbsp;%s&nbsp;</td>",m_type[room.type] ? m_type[room.type] -> name : "&lt;invalid type&gt;");
			printf("<td bgcolor=#c0c0c0>&nbsp;%s&nbsp;</td>",m_size[room.size] ? m_size[room.size] -> name : "&lt;invalid size&gt;");
			printf("</tr>");
		}
		q.free_result();
	}
	printf("<br>");

	if (m_edit)
	{
		db::Room room(db,m_edit);
		// exits
		if (!room.virtual_room)
		{
			{
			Table tbl("#000000", 3, "&nbsp;Orientation");

			// - - n - u
			// - - N U -
			// w W - E e
			// - D S - -
			// d - s - -
			for (int y = 0; y < 5; y++)
			{
				printf("<tr bgcolor=#ffffc0>");
				for (int x = 0; x < 5; x++)
				{
					int dir = -1;
					printf("<td align=middle valign=middle>");
					switch (y * 5 + x)
					{
					case 2: // room north
						dir = 0;
						break;
					case 4: // room up
						dir = 4;
						break;
					case 7: // NORTH
						printf("&nbsp;<b>North</b>&nbsp;");
						break;
					case 8: // UP
						printf("&nbsp;<b>Up</b>&nbsp;");
						break;
					case 10: // room west
						dir = 3;
						break;
					case 11: // WEST
						printf("&nbsp;<b>West</b>&nbsp;");
						break;
					case 12: // this room
						dir = 6;
						break;
					case 13: // EAST
						printf("&nbsp;<b>East</b>&nbsp;");
						break;
					case 14: // room east
						dir = 1;
						break;
					case 16: // DOWN
						printf("&nbsp;<b>Down</b>&nbsp;");
						break;
					case 17: // SOUTH
						printf("&nbsp;<b>South</b>&nbsp;");
						break;
					case 20: // room down
						dir = 5;
						break;
					case 22: // room south
						dir = 2;
						break;
					}
					if (dir == 6)
					{
						printf("<div style=\"background-color: #808080\">");
						ShowRoom(room,false,false,true);
						printf("</div>");
						printf("<b>%ld</b>",room.num);
					}
					else
					if (dir > -1)
					{
						char sql[200];
						sprintf(sql,"select * from room_door where room=%ld and direction=%d",
							room.num,dir);
						db::Room_door x(db,sql);
						if (x.toroom)
						{
							db::Room room(db,x.toroom);
							printf("<div style=\"background-color: #808080\">");
							if (room.num)
							{
								ShowRoom(room,true,false,true);
							}
							else
							{
								// virtual zone?
								sprintf(sql,"select * from vzone where vnumstart<=%ld and %ld<vnumstart+sizex*sizey",x.toroom,x.toroom);
								db::Vzone vz(db,sql);
								if (vz.num)
								{
									ShowVirtual(vz,true,false,true);
								}
							}
							printf("</div>");
						}
						printf("<form action=%s method=post>",GetCgiName().c_str());
						printf("<input type=hidden name=roomform value=orientation>");
						printf("<input type=hidden name=room value=%ld>",room.num);
						printf("<input type=hidden name=direction value=%d>",dir);
						printf("&nbsp;<input type=text size=9 name=toroom value=%ld>&nbsp;",x.toroom);
						printf("</form>");
					}
					printf("</td>");
				}
				printf("</tr>");
			} // for (dirs)
			} // end of tbl
			printf("<br>");
		} // if !virtual_room

		{
			Table tbl("#000000", 3);
			long affects = asciiconvert(room.affects);
		
			printf("<form action=%s method=post>",GetCgiName().c_str());
			printf("<input type=hidden name=onum value=%ld>",room.num);

			inputnum("Vnum","num",room.num);
			inputstr("Name","name",60,room.name,true);
			textarea("Description","descr",room.description,true);
			inputselect(q,"Zone","zone","select num,name from zone order by name",room.zone,true);
			// affects
			inputbit(q,"Affects","affects","select num,name from room_bit",affects,4);
			inputselect(q,"Type","type","select num,name from room_type",room.type);
			inputselect(q,"Size","size","select num,name from room_size",room.size);
			printf("<tr bgcolor=#ffffc0>");
			printf("<td>VirtualRoom</td>");
			printf("<td><input type=checkbox name=virtual_room value=1%s>",
				room.virtual_room ? " CHECKED" : "");
			printf("</td>");
			printf("</tr>");
			inputstr("VirtualChar","virtual_char",20,room.virtual_char,true);
			inputstr("DesignChar","design_char",2,room.design_char);
			inputselect(q,"Resource","resource","select num,alias from resource order by alias",room.resource,true);
			inputselect(q,"ResourceSpeed","resource_speed","select num,name from resource_speed order by num",room.resource_speed);

			inputsubmit(" Save ");

			printf("</form>");
		}
		printf("<br>");
/*
CREATE TABLE room_door (
  num int(11) NOT NULL auto_increment,
  room int(11) NOT NULL default '0',
  direction int(11) NOT NULL default '0',
  keyword varchar(255) NOT NULL default '',
  Type varchar(255) NOT NULL default '',
  vnumkey int(11) NOT NULL default '0',
  ToRoom int(11) NOT NULL default '0',
  PRIMARY KEY  (num),
  KEY room (room,direction)
) TYPE=MyISAM;
*/
		// door
		if (!room.virtual_room)
		{
		sprintf(sql,"select * from room_door where room=%ld",room.num);
		q.get_result(sql);
		while (q.fetch_row())
		{
			db::Room_door door(db,&q);
			{
				Table tbl("#000000", 3);
				long type = asciiconvert(door.type);

				printf("<form action=%s method=post>",GetCgiName().c_str());
				printf("<input type=hidden name=num value=%ld>",door.num);
				printf("<input type=hidden name=roomnum value=%ld>",room.num);

				inputselect(q2,"Direction","dir","select num,name from room_dir order by ordning",door.direction);
				inputstr("Keyword","keyword",40,door.keyword);
				inputbit(q2,"Type","type","select num,name from door_type",type,3,true);
				inputnum("Key","key",door.vnumkey);
				inputnum("ToRoom","toroom",door.toroom);
				inputsubmit(" Save Door ");

				printf("</form>");
			}
			printf("<br>");
		}
		q.free_result();

		// new door
		{
			Table tbl("#000000", 3);
			long type = 1;

			SetColor("#c0ffff");
			printf("<form action=%s method=post>",GetCgiName().c_str());
			printf("<input type=hidden name=num value=%d>",0);
			printf("<input type=hidden name=roomnum value=%ld>",room.num);

			inputselect(q2,"Direction","dir","select num,name from room_dir order by ordning",0);
			inputstr("Keyword","keyword",40,"");
			inputbit(q2,"Type","type","select num,name from door_type",type,3,true);
			inputnum("Key","key",-1);
			inputnum("ToRoom","toroom",-1);
			inputsubmit(" Add Door ");

			printf("</form>");
		}
		printf("<br>");
		} // virtual room

		// extra
		sprintf(sql,"select * from room_extra where room=%ld",room.num);
		q.get_result(sql);
		while (q.fetch_row())
		{
			db::Room_extra x(db,&q);
			{
				Table tbl("#000000", 3);
				
				printf("<form action=%s method=post>",GetCgiName().c_str());
				printf("<input type=hidden name=num value=%ld>",x.num);
				printf("<input type=hidden name=roomnum value=%ld>",room.num);

				inputstr("Keyword","keyword",60,x.keyword);
				textarea("Description","descr",x.description,true);
				inputsubmit(" Save Extra ");

				printf("</form>");
			}
			printf("<br>");
		}
		q.free_result();
		
		// new extra
		{
			Table tbl("#000000", 3);
			
			SetColor("#c0ffff");
			printf("<form action=%s method=post>",GetCgiName().c_str());
			printf("<input type=hidden name=num value=%d>",0);
			printf("<input type=hidden name=roomnum value=%ld>",room.num);

			inputstr("Keyword","keyword",60,"");
			textarea("Description","descr","");
			inputsubmit(" Add Extra ");

			printf("</form>");
			SetColor("#ffffc0");
		}
		printf("<br>");
	}
}


} // namespace cgi


/*
CREATE TABLE room (
  num int(11) NOT NULL default '0',
  name varchar(255) NOT NULL default '',
  description text NOT NULL,
  Zone int(11) NOT NULL default '0',
  Affects varchar(255) NOT NULL default '',
  Type int(11) NOT NULL default '0',
  Size int(11) NOT NULL default '0',
  PRIMARY KEY  (num)
) TYPE=MyISAM;
CREATE TABLE room_extra (
  num int(11) NOT NULL auto_increment,
  room int(11) NOT NULL default '0',
  keyword varchar(255) NOT NULL default '',
  description varchar(255) NOT NULL default '',
  PRIMARY KEY  (num),
  KEY room (room,keyword)
) TYPE=MyISAM;
*/

