//////////////////////////////////////////////////////////////
//Wolfshade MUD server
//Copyright (C) 1999 Demetrius and John Comes
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////

#include "stdinclude.h"
// Database includes
#include <mysql/mysql.h>
#include <libsql++.h>
#include <libwolfshade.h>
//
#include "CGame.h"
#include "main.h"

#define DEB(x) x; fflush(stdout);


/*
-classic Boot the world using the "old" textfiles
-generatedb Populate the database using the "old" textfiles

Without any of those two: Boot the world using the database

Two new fields in mob/obj/room/zone(/shop?):
disable
loaded

"disable" tells the server to load the object or not
"loaded" is reset when the server starts, then set when the object is loaded
- this way objects can be added "on the fly" and loaded at runtime
... (select * from object where not disable and not loaded)...

Fixa p!
/ah

*/


int main(int argc, char *argv[])
{
	const char *strUsage = 
		"Wolfshade mud options: \r\n" 
		"-classic\r\n"
		"-generatedb\r\n"
		"-nocolor\r\n"
		"-n name server true [default: false] turn on if your network has a name server\r\n"
		"-h this help menu\r\n"
		"-p set port number mud will run on [default: 6969]\r\n"
		"-z set zone index to load [default: zoneindex]\r\n"
		"-o set object index to load [default: objectindex]\r\n"
		"-r set realroom index to load [default: realroomindex]\r\n"
		"-m set mob index to load [default: mobindex]\r\n"
		"-v verion\r\n"
		"report bugs, comments, sugggestions, flames to: \r\n"
		"duth@wolfshade.com or kythis@wolfshade.com";
	try
	{
		MyString strOpts;
		register int i;
		for(i=0;i<argc;i++)
		{
			strOpts.Format("%s %s",
				(char *)strOpts,
				argv[i]);
		}
		if(strOpts.Find("-h")!=-1)
		{
//			std::cout << (char *)strUsage << endl;
			printf("%s\n",(char *)strUsage);
		}
		else if(strOpts.Find("-v")!=-1)
		{
#ifdef _WOLFSHADE_WINDOWS_
			std::cout << "1.64.x" << endl;
#else
//			std::cout << VERSION << endl;
			printf("%s\n",(char *)VERSION);
#endif
		}
		else
		{
			//CMud Time must be first
			//all timing for the mud is done in CMudTime 
			//so they are not initialized until CMudTime
			//is created.  //Therefore no timing stuff can be in
			//cGRR or GVM!
			CMudTime MudTime;
			CGame Wolfshade(strOpts);

			if (!Wolfshade.GenerateDB())
			{
				Wolfshade.Heartbeat(MudTime);
			}
			DEB(printf("Leaving main()...\n");)
		}
	}
	catch(CError *Er)
	{
		ErrorLog << Er->GetMessage() << endl;
		fflush(stdout);
		fflush(stderr);
		CCharacterAttributes::CleanUp();
		return 100;
	}
	catch(...)
	{
		ErrorLog << "Unknown error caught in main()..." << endl;
		fflush(stdout);
		fflush(stderr);
		CCharacterAttributes::CleanUp();
		return 200;
	}
	CCharacterAttributes::CleanUp();
	return 0;
}

