/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <vector>
#include <string>
#include <libcgi++.h>


typedef enum {
	ST_NOCLASS = 0,
	ST_BEGIN,
	ST_INCLASS
} State;

typedef enum {
	NONE = 0,
	PUBLIC,
	PROTECTED,
	PRIVATE
} PPP;


void warning(char *file,int line,char *text)
{
	printf("%s line %d:  %s\n",file,line,text);
}


void xprintf(FILE *f,char *format,const char *s)
{
	if (*s)
	{
		if (*s == 9)
			fprintf(f,"  %s\n",s + 1);
		else
			fprintf(f,format,s);
	}
}


void write_vec(FILE *fil2,vector<string> vv)
{
	for (vector<string>::iterator it = vv.begin(); it != vv.end(); it++)
	{
		string tmp = *it;
		if (tmp.size())
		{
			xprintf(fil2,"%s\n",tmp.c_str());
		}
	}
}


void write_class(FILE *fil2,vector<string> &pub,
	vector<string> &pub_st,
	vector<string> &pro,
	vector<string> &pro_st,
	vector<string> &pri,
	vector<string> &pri_st)
{
	fprintf(fil2,"\npublic:\n");
	write_vec(fil2,pub);
	if (pub.size())
		fprintf(fil2,"\n");
	write_vec(fil2,pub_st);

	fprintf(fil2,"\nprotected:\n");
	write_vec(fil2,pro);
	if (pro.size())
		fprintf(fil2,"\n");
	write_vec(fil2,pro_st);

	fprintf(fil2,"\nprivate:\n");
	write_vec(fil2,pri);
	if (pri.size())
		fprintf(fil2,"\n");
	write_vec(fil2,pri_st);

	while (pub.size())
	{
		pub.erase(pub.begin());
	}
	while (pub_st.size())
	{
		pub_st.erase(pub_st.begin());
	}
	while (pro.size())
	{
		pro.erase(pro.begin());
	}
	while (pro_st.size())
	{
		pro_st.erase(pro_st.begin());
	}
	while (pri.size())
	{
		pri.erase(pri.begin());
	}
	while (pri_st.size())
	{
		pri_st.erase(pri_st.begin());
	}
}


int main(int argc,char *argv[])
{
	FILE *fil;
	FILE *fil2;
	State st;
	PPP pp = PRIVATE;
	int line;
	int x;
	char *s;
	bool bCopy = false;
	char tmp[300];
	char slask[300];
	char current_class[300];

	if (argc == 1)
	{
		fprintf(stderr,"Usage: %s [-copy] <file[s]>\n",*argv);
		exit(-1);
	}

	for (int i = 1; i < argc; i++)
	if (!strcmp(argv[i],"-copy"))
	{
		bCopy = true;
	}
	else
	{
		vector<string> pub;
		vector<string> pro;
		vector<string> pri;
		vector<string> pub_st;
		vector<string> pro_st;
		vector<string> pri_st;

		st = ST_NOCLASS;

		if ((fil = fopen(argv[i],"rt")) != NULL)
		{
			printf("%s\n",argv[i]);

			strcpy(tmp,argv[i]);
			strcat(tmp,".new");
			fil2 = fopen(tmp,"wt");
//			fprintf(fil2,"//%%! Reordered automatically by 'reorder'\n");
			s = fgets(slask,300,fil);
			line = 1;
			if (!strncmp(slask,"//%!",4))
			{
				printf("Already reordered - remove first line to do this anyway\n");
			}
			else
			while (!feof(fil) || s)
			{
#define C slask[strlen(slask) - 1]
				while (strlen(slask) && (C == 13 || C == 10 || C == ' ' || C == 9))
				{
					C = 0;
				}
				strcpy(tmp,slask); // save original
				if ((s = strstr(slask,"//")) != NULL)
				{
					*s = 0;
				}
				x = 0;
				while (slask[x] == ' ' || slask[x] == 9)
				{
					x++;
				}
				//
				switch (st)
				{
				case ST_NOCLASS:
					if (!strncmp(slask,"class ",5) && !strstr(slask,";") )
					{
						Parse pa(slask);
						pa.getword(); // 'class'
						pa.getword( current_class );
						printf(" %s\n",current_class);
						st = ST_BEGIN;
						fprintf(fil2,"\n%s\n",tmp);
					}
					else
					if (strncmp(tmp,"//#include",10))
					{
						fprintf(fil2,"%s\n",tmp);
					}
					break;
				case ST_BEGIN:
					fprintf(fil2,"%s\n",tmp);
					if (*slask != '{')
					{
						warning(argv[i],line,"No '{' in beginning");
					}
					st = ST_INCLASS;
					pp = NONE;
					break;
				case ST_INCLASS:
					if (!strncmp(slask,"public:",7))
					{
						pp = PUBLIC;
					}
					else
					if (!strncmp(slask,"protected:",10))
					{
						pp = PROTECTED;
					}
					else
					if (!strncmp(slask,"private:",8))
					{
						pp = PRIVATE;
					}
					else
					if (*slask == '}')
					{
						write_class(fil2, pub, pub_st, pro, pro_st, pri, pri_st);
						fprintf(fil2,"%s\n",tmp);
						st = ST_NOCLASS;
					}
					else
					{
						bool bStructor = false;
						if ((s = strstr(slask,current_class)) != NULL &&
							s[strlen(current_class)] == '(' &&
							(s = strstr(s,")")) != NULL &&
							strstr(s,";") )
						{
							bStructor = true;
							if ((s = strstr(slask,"{")) != NULL)
							{
								if (!strstr(s,"}"))
								{
									bStructor = false;
								}
							}
							if (!bStructor)
							{
								printf(" %s\n",tmp);
							}
						}
						if (!strncmp(slask + x,"struct",6))
						{
						}
						if (!strncmp(slask + x,"static",6))
						{
						}
						switch (pp)
						{
						case NONE:
							xprintf(fil2,"%s\n",tmp);
							break;
						case PUBLIC:
							if (bStructor)
								pub.insert(pub.end(), tmp);
							else
								pub_st.insert(pub_st.end(), tmp);
							break;
						case PROTECTED:
							if (bStructor)
								pro.insert(pro.end(), tmp);
							else
								pro_st.insert(pro_st.end(), tmp);
							break;
						case PRIVATE:
							if (bStructor)
								pri.insert(pri.end(), tmp);
							else
								pri_st.insert(pri_st.end(), tmp);
							break;
						}
					}
					break;
				}
				//
				s = fgets(slask,300,fil);
				line++;
			}
			fclose(fil);
			fclose(fil2);
			printf("\n");
			if (bCopy)
			{
				char old[300];
				strcpy(tmp,argv[i]);
				strcat(tmp,".new");
				strcpy(old,argv[i]);
				strcat(old,".old");
				rename(argv[i], old);
				rename(tmp, argv[i]);
			}
		}
	}
}



